/*
 *  Formula-Calc.h
 *  Version 1.5.1 / 980129
 *  by Satoshi Takahashi and Jiro Kikuchi
 */


#ifndef FORMULA_CALC_H  /*pragma once*/
#define FORMULA_CALC_H


/**************************** Expand and Sort **********************************/


void SortFracFormula(void);

void SortFracNumFormula(int fNum);

void ExpandFB(s_f_FB** orgFBP, s_f_FB** retLastFBP,s_f_FB** denomFBP);
void ExpandTopBase(s_f_FB* curFB, s_f_FB** newTopFBP, s_f_FB** newLastFBP);
void ExpandTopBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP);
void ExpandNoFracOneBase(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP);
void ExpandNoFracOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP);
void ExpandFracOneBase(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP);
void ExpandFracOneBlock(s_f_FB* curFB, s_f_FB** newTopFBP,
					s_f_FB** newLastFBP, s_f_FB** newDenomFBP);
void ExpandPowerBlock(s_f_FB** newTopFBP, s_f_FB** newLastFBP,
								s_f_FB** newDenomFBP, int power);

void SortFormulaBlock(s_f_FB **fbp);


/********************************* Truncate ***********************************/

void TruncateFormula(void);
int TruncateCheckInput(int inputNum, char *formulaString,
							char *symbolString, int power);
void TruncateFB(s_f_FB **fbp, int symbolNum, int power);
void TruncateTopFB(s_f_FB **fbp, int symbolNum, int power);


/**************************** Sum Truncate ***********************************/

void SumTruncateFormula(void);
int GetSumTruncParams(char *inputline, int *formulaNumP, int *symbolSumP, int *symbolNumArray, int *totalPowerP);
void SumTruncateFB(s_f_FB **inFBP, int symbolSum, int *symbolNumArray, int power);
void SumTruncateTopFB(s_f_FB **fbp, int symbolSum, int *symbolNumArray, int power);
int SumPower(int *powerArray, int symbolSum, int *symbolNumArray);


/**************************** Expand Denominator ******************************/

void ExpandDenominator(void);
void EDFrac(s_f_FB **numFBP, s_f_FB **denomFBP, int symbolNum, int power);
int DivideDenom(s_f_FB *denomFB, int symbolNum, s_f_FB **retDenom0FBP, s_f_FB **retDenom1FBP);
s_f_FB *MakeNum2(s_f_FB *denomFB, s_f_FB *denom0, int symbolNum, int power);
void EDMakeSeries(s_f_FB *curNewFB, s_f_FB *curDenomFB, s_f_FB *denom0,
					int power, int symbolNum);


/**************************** Reduce Fraction ******************************/

void ReduceFormula(void);
int GetReduceParams(char *inputline, int *formulaNumP,
			int *symbolNumP, int *powerP,
			int *coefFlagP, double *coefP);
int ReduceFB(s_f_FB **inFBP, int symbolNum, int power, s_f_FB **curFBP);
void UndoReduceFB(s_f_FB **inFBP, int symbolNum, int power, s_f_FB *curFB);
int ReduceCoefFB(s_f_FB **inFBP, double coef);


/**************************** Select Powers ******************************/

void SelectCoefFormula(int coefFlag);
int SelectFB(s_f_FB **inFBP, int symbolNum, int power, int coefFlag);
void SelectTopFB(s_f_FB **fbp, int symbolNum, int power, int coefFlag);


/**************************** Get Numerator and Denominator ******************************/

void GetNumerator(void);
void GetDenominator(void);

/**************************** Set Compare Order ***********************************/
void SetCompareOrder(void);
int GetSetCompareOrderParams(char *inputline,
						int *symbolSumP, int *symbolNumArray);

/**************************** Substitute ***********************************/

void SubstituteFormula(void);
void SubsFB(s_f_FB **inFB, int symbolNum, int subsFNum);


#if TEST_CO
/**************************** Copy Formula ******************************/

void CopyFormula(void);
void ReadAndAddFormula(int *fNumP);
#endif /*TEST_CO*/

/**************************** Collect Formula ******************************/

void CollectFormula(void);
int CollectFB (s_f_FB **retFBP, int symbolNum);

int GetCollectArrayFB(s_f_FB *inFB, s_f_FB ***collectFBArrayP,
						int symbolNum, int *maxPowerP);

int MaxPowerFB(s_f_FB *inFB, int symbolNum, int *maxPowerP);


/**************************** Solve Formula ******************************/

void SolveFormula(void);


#if TEST_DIV
/**************************** Polynomial Division ******************************/

void DivideFormula(void);
void FactorFormula(void);
void DivideOrFactorFormula(int factorFlag);
int GetParamsFnumTermFBS(int *fNumP, s_f_FB **termFBP, s_f_FB **divFBP, int *sNumP, int *powerP);
int CheckGetSNumPower(s_f_FB *termFB, int *sNumP, int *powerP);
int DivideFB( s_f_FB **inFBP, int sNum, int power, s_f_FB *divFB, s_f_FB **redFBP);
int ExpandAndSortFB(s_f_FB **fbp);

#endif /*TEST_DIV*/

/**************************** Ciean complex ******************************/

int CleanComplexFB(s_f_FB **inFBP, int symbolNum);
void CleanComplexFormula(void);
void CompMakeDenomReal(void);


/**************************** Differentiate ******************************/

#if TEST_DIFF
void DifferentiateFormula(void);
int DifferentiateFB(s_f_FB **inFBP, int symbolNum);
void DifferentiateTopFB(s_f_FB **fbp, int symbolNum);

#endif /*TEST_DIFF*/


/**************************** Polynomial Reduce ******************************/

#if TEST_PRED
void PolynomialReduceFormula(void);
#endif
#endif /*pragma once*/