/*
 *  Formula-IO.c
 *  Version 1.9.1 / 010219
 *  by Satoshi Takahashi and Jiro Kikuchi
 */

#ifndef FORMULA_IO_C /* pragma once */
#define FORMULA_IO_C

/************************ Switches *******************/

#include <Formula-Switches.h>

/************************ Include Headers *******************/
#if S_UNIX
#include <s_Toolbox.h>
#endif

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>

#include <Formula.h>
#include <Formula-v.h>
#include <Formula-Common.h>

#include <StartSCL.h>
#include <s_Common.h>
#include <DialogFOpen.h>
#include <s_String.h>
#include <s_Print.h>
#include <s_RedrawFunc.h>
#include <s_Debug.h>
#include <DragAndDrop.h>


/************************ Input Command Line *******************/

int ReadOneLine(void)
{
	if (s_fgets(s_f_inputLine,S_F_MAX_INPUT_LENGTH,s_f_input)) /* not eof */
	{
		if (s_f_inputLine[strlen(s_f_inputLine)-1] == '\n' )
		{
			CheckPause();
			s_f_inputLine[strlen(s_f_inputLine)-1] = 0; /* s_f_inputLine made not to contain \n at last line */
			s_f_contLineFlag = 0;
		}
		else /* input line continues */
		{
			s_f_contLineFlag = 1; /* line continues */
		}
		s_f_str = s_f_inputLine;
		s_f_strEnd = s_f_inputLine + S_F_MAX_INPUT_LENGTH-1;
		return 1; /* not eof */
	}
	else /* already reached eof */
	{
		SetFormulaInputConsole();
		s_f_contLineFlag = 0;
		return 0; /* eof */
	}
}


/************************ Output Formula on Display *******************/

void PrintFormula(void)
{
int i;
int saveShowFlag;

	SkipSpaces(&s_f_str);
	i = FormulaNumber(s_f_str);
	if(i >= s_f_fNum)
	{
		s_printf("Formula %s is not defined.\n",s_f_str);
	}
	else
	{
		saveShowFlag = s_f_showFlag;
		s_f_showFlag = 1;
		PrintNumFormula(i);
		s_f_showFlag = saveShowFlag;
	}
}

int s_f_showFlag = 1;

void PrintNumFormula(int n)
{
s_f_FB *tmpBlock;

	dwf("PrintNumFormula 0\n");
	if (!s_f_showFlag)
		return;

	if(n >= s_f_fNum)
	{
		s_printf("Formula NO. %d is out of range.\n",n);
	}
	else
	{
		s_f_curFNum = n;
		SetFont(s_f_defaultFont);
		
		ScrollFormula(2);
		ScrollFormula(min(s_f_formulas[n].divDepth,s_f_vDivLevel));
		s_MoveTo(0,s_f_fy);
		
		DrawFormulaCString(" ");
		DrawStyleText(s_f_formulas[n].printName);
		if (s_f_saveFlag) fprintf(s_f_formulaOutFile,s_f_fName[n]);
		DrawFormulaCString(" =");

		if (s_f_formulas[n].fb->power!=1)
		{
			tmpBlock = NewFormulaBlock();
			tmpBlock->type = BlockType;
			tmpBlock->block = s_f_formulas[n].fb;
			PrintFormulaBlock(tmpBlock,0);
		}
		else
			PrintFormulaBlock(s_f_formulas[n].fb,0);
		
		if (s_f_saveFlag) fprintf(s_f_formulaOutFile,"\n");
		ScrollFormula(min(s_f_formulas[n].divDepth,s_f_vDivLevel));
	}
	dwf("PrintNumFormula END\n");
}

void PrintFormulaBlock(s_f_FB *blockP, int divLevel)
{
int vDivFlag = 0; /* 0: not in  1: in upper  2: in lower */
int topFlag = 1;
Point divLPt, divRPt;

int rewritingFlag = 0;
Point prevPenLoc;
int orgSaveFlag;


	dwf("PrintFormulaBlock 0\n");

	orgSaveFlag = s_f_saveFlag;

	while(blockP)
	{
		CheckRightMargin();

		GetPen(&prevPenLoc);
		
		PrintOperand(blockP, topFlag, &vDivFlag, &divRPt, divLPt);
		
		topFlag = 0;

		if ((divLevel < s_f_vDivLevel)&&(blockP->next)
					&& (blockP->next->operand == DivOp))
			VDivUpper(&vDivFlag, &divLevel, &divLPt);
		
		switch (blockP->type)
		{
		case BaseType:
			PrintFormulaBase(blockP->base);
			break;
		case BlockType:
			if (CheckPrintParen(blockP, vDivFlag))
			{
				DrawFormulaCString(" (");
				PrintFormulaBlock(blockP->block,divLevel);
				DrawFormulaCString(" )");
				if (blockP->block->power!=1)
					PrintPower(blockP->block->power);
			}
			else
			{
				if (s_f_saveFlag && vDivFlag)
					fprintf(s_f_formulaOutFile, " (");
				PrintFormulaBlock(blockP->block,divLevel);
				if (s_f_saveFlag && vDivFlag)
					fprintf(s_f_formulaOutFile, " )");
			}
			break;
		}
		
		if (vDivFlag == 2)
			VDivEnd(&vDivFlag, &divLevel, divRPt, divLPt);


		if ( rewritingFlag )
		{
			rewritingFlag =0;
			s_f_saveFlag = orgSaveFlag;
			blockP = blockP->next;
		}
		else if ( CheckRightBound() )
		{
			EraseFormulaBlock(prevPenLoc);
			ScrollFormula(2);
			MoveTo(s_f_contLeft,s_f_fy);
			rewritingFlag = 1;
			s_f_saveFlag = 0;
			s_f_rightMargin += 30;
		}
		else
		{
			blockP = blockP->next;
		}			
	}
	dwf("PrintFormulaBlock END\n");
}


void PrintOperand(s_f_FB *blockP, int topFlag, int *vDivFlagP,
						Point *divRPtP, Point divLPt)
{
	dwf("PrintOperand 0\n");

	switch(blockP->operand)
	{
	case AddOp:
		if (!topFlag)
			DrawFormulaCString(" +");
		break;
	case SubOp:
#if !S_UNIX
		DrawCString(" "); /*  and - are different */
#else
		DrawStyleText(" $Symbol$-"); 
#endif
		if (s_f_saveFlag) fprintf(s_f_formulaOutFile, " -");
		break;
	case MulOp:
		if (!topFlag)
		{
			DrawStyleText(" $Symbol$"); /* Multiply mark X */
			SetFont(s_f_defaultFont);
			if (s_f_saveFlag) fprintf(s_f_formulaOutFile, " *");
		}
		break;
	case DivOp:
		if ((*vDivFlagP!=1)
				||((blockP->next)&&(blockP->next->operand == DivOp)))
			DrawFormulaCString(" /");
		else
		{
			if (s_f_saveFlag) fprintf(s_f_formulaOutFile, " /");
			VDivLower(vDivFlagP, divLPt, divRPtP);
		}
		break;
	}
	dwf("PrintOperand END\n");
}

void VDivUpper(int *vDivFlagP, int *divLevelP, Point *divLPtP)
{
	*vDivFlagP = 1;
	(*divLevelP)++;
	GetPen(divLPtP);
	s_Move(2,- s_f_divVMove);
}

void VDivLower(int *vDivFlagP, Point divLPt, Point *divRPtP)
{
	*vDivFlagP = 2;
	Move(0,s_f_divVMove);
	GetPen(divRPtP);
	MoveTo(divLPt.h+2,divLPt.v+s_f_divVMove);
}

void VDivEnd(int *vDivFlagP, int *divLevelP, Point divRPt, Point divLPt)
{
Point divRPt2;

	*vDivFlagP = 0;
	(*divLevelP)--;
	Move(0,- s_f_divVMove);
	GetPen(&divRPt2);
	MoveTo(divLPt.h+2,divLPt.v - s_f_divVMove+2);
	LineTo(max(divRPt.h,divRPt2.h),divLPt.v - s_f_divVMove+2);
	Move(2,s_f_divVMove-2);
}


int CheckPrintParen(s_f_FB *blockP, int vDivFlag)
{
	if (s_f_showBlock || (blockP->block->power!=1))	return 1;
	else if (vDivFlag)								return 0;
	else if (blockP->operand == DivOp)				return 1;
	else if ((blockP->operand == MulOp)
				&&((blockP->block->operand == AddOp)
					||(blockP->block->operand == SubOp)))
													return 1;
	else											return 0;
	
}

void PrintPower(int power)
{
	Move(0,-5);
	s_TextSize(s_f_fontSize-2);
	s_DirectDprintf("%d",power); /* print at current poin */
	TextSize(s_f_fontSize);
	Move(0,5);
	
	if (s_f_saveFlag) fprintf(s_f_formulaOutFile,"^%d",power);
}


void PrintFormulaBase(s_f_Base *base)
{
int i;
int oneFlag = 1;

	dwf("PrintFormulaBase 0\n");

	if (base->coef != 1)
	{
		oneFlag = 0;
		if (base->coef > 1.0E+18)
			s_DirectDprintf(" %.18lE", base->coef);
		else
			s_DirectDprintf(" %.*f", s_f_coefDecimalNum, base->coef);
		if (s_f_saveFlag)
			fprintf(s_f_formulaOutFile, " %.*f", s_f_coefDecimalNum, base->coef);
	}
	for(i=0;i<s_f_sNum;i++)
	{
		if (base->power[i])
		{
			oneFlag = 0;
			DrawFormulaCString(" ");
#if 1
				DrawStyleText(s_f_symbolTable[i].printName);
#else
			if (s_f_symbolTable[i].controlFlag)
				DrawStyleText(s_f_symbolTable[i].printName);
			else
				DrawStyleText(s_f_sName[i]);
#endif /*else 1*/
			if (s_f_saveFlag)
				fprintf(s_f_formulaOutFile,s_f_sName[i]);
			if (base->power[i]!=1)
				PrintPower(base->power[i]);
		}
	}
	if (oneFlag)
		DrawFormulaCString(" 1");

	dwf("PrintFormulaBase END\n");
}

void ScrollFormula(int n)
{
int i;
Rect r;

	for (i = 0; i< n; i++)
	{
		CheckPause();
		if (s_f_fy < (DrawingHeight() - s_f_bottomMargin))
		{
			s_f_fy += s_f_divVMove;
		}
		else
		{
			if (s_f_printFlag)
			{
				if (s_f_printContFlag)
				{
					EndPrintingBlind();
				}
				else
				{
					s_f_printFlag = EndPrinting();
					s_f_printContFlag = 1;
				}
			}
			if (s_f_printFlag) /* rewritten by EndPrinting */
			{
				ClearFormulaDisp();
				s_f_fy += 2*s_f_divVMove;
				BeginPrinting();
			}
			else
			{
				ScrollRect(SCWindowBounds(s_displayWindow),0,-s_f_divVMove*s_f_multiScroll,0);
				SetRect(&r,0,DrawingHeight() - s_f_divVMove*s_f_multiScroll,
										DrawingWidth(),DrawingHeight());
				EraseRect(&r);
				s_f_fy -= s_f_divVMove*(s_f_multiScroll-1);
			}
		}
	}
}

void CheckRightMargin(void)
{
Point penLoc;

	dwf("CheckRightMargin 0\n");

	GetPen(&penLoc);
	if (penLoc.h > (DrawingWidth() - s_f_rightMargin))
	{
		ScrollFormula(2);
		MoveTo(s_f_contLeft,s_f_fy);
		s_f_rightMargin--;
	}
	dwf("CheckRightMargin END\n");
}


void ClearFormulaDisp(void)
{
	EraseDrawing();
	s_f_fy = 0;
}

/**************************** Save Formula ***********************************/

void BeginFormulaSaving(void)
{

	SkipSpacesAndReadCont();

	if (s_f_str&&s_f_str[0])
	{
		s_f_formulaOutFile = fopen(s_f_str,"w");
	}
	else
		s_f_formulaOutFile = DialogFOpen("formula-output","w");
	if (s_f_formulaOutFile)
		s_f_saveFlag = 1;
}

void EndFormulaSaving(void)
{
	fclose(s_f_formulaOutFile);
	s_f_saveFlag = 0;
}

void DrawFormulaCString(char *str)
{
	if (s_f_saveFlag)
		fprintf(s_f_formulaOutFile, str);
	DrawCString(str);
}

/*-------------------- Rewrite --------------------*/

int CheckRightBound(void)
{
Point penLoc;

	GetPen(&penLoc);
	if (penLoc.h >= DrawingWidth())
	{
		return 1;
	}
	else
		return 0;
}

void EraseFormulaBlock(Point prevPenLoc)
{
Point curPenLoc;
Rect r;

	GetPen(&curPenLoc);
	SetRect(&r, prevPenLoc.h, prevPenLoc.v - 1.5*s_f_divVMove, curPenLoc.h, prevPenLoc.v + 0.5*s_f_divVMove);
	EraseRect(&r);
}


/**************************** Input Formula ***********************************/

void SetFormulaByString(void)
{
char LHS[S_F_MAX_NAME_LENGTH+1];
int fn;
char printFName[S_F_MAX_NAME_LENGTH+1];
#if !TEST_CO
int controlFlag;
int controlNum = 0;
int controlType;
#endif

#if TEST_CO
s_f_ControlPB fControlPB;
#endif

	dwf("SetFormulaByString 0\n");
	if (!s_f_setFlag)
		InitilaSetSetUp();
	
	SkipSpacesAndReadCont();

	if(!(isalpha(s_f_str[0]) || s_f_str[0] == '\\'))
	{
		s_printf("In SetFormulaByString:\nIllegal character '%c'\n",s_f_str[0]);
		return;
	}

	if (s_f_inputShowFlag)
	{
		ScrollFormula(2);
		MoveTo(0, s_f_fy);
		DrawStyleText(s_f_str);
		ScrollFormula(0);
	}

#if TEST_CO
	if ( ReadControlOrSymbolName(LHS, printFName, &fControlPB) )
		return;

	SkipSpacesAndReadCont();

	if(s_f_str[0]!='=') 
	{
		s_printf("In SetFormulaByString:\n'=' expected.\n");
		return;
	}

	if (AddFormula(LHS, &fn, printFName, fControlPB.usedFlag) )
		return;
#else
	if ( ReadControlOrSymbolName(LHS, printFName, &controlNum, &controlFlag) )
		return;

	SkipSpacesAndReadCont();

	if(s_f_str[0]!='=') 
	{
		s_printf("In SetFormulaByString:\n'=' expected.\n");
		return;
	}

	if (AddFormula(LHS, &fn, printFName, controlFlag) )
		return;
#endif


	SFStrInc();
	SkipSpacesAndReadCont();
	
	
	if ( ReadFormulaBlock(&(s_f_formulas[fn].fb), &(s_f_formulas[fn].divDepth)) )
	{
		/*free(to be made)*/
		/* recover code ?? */
		return ; /* Error! */
	}
	SkipSpacesAndReadCont();

	if (s_f_str[0])
	{
		printf("In SetFormulaByString:\nIllegal character '%c'\n",s_f_str[0]);
		return;
	}

	PrintNumFormula(fn);
	dwf("SetFormulaByString END\n");
}

void InitilaSetSetUp(void)
{
	s_f_setFlag = 1;

	if (!(s_f_sName = calloc(s_f_symbolMax, sizeof(char*))))
		s_OOMError();
	if (!(s_f_symbolTable = calloc(s_f_symbolMax, sizeof(s_f_symbolTableStruct))))
		s_OOMError();
	if (!(s_f_compareOrder = calloc(s_f_symbolMax, sizeof(int))))
		s_OOMError();

	if (!(s_f_fName = calloc(s_f_formulaMax, sizeof(char*))))
		s_OOMError();
	if (!(s_f_formulas = calloc(s_f_formulaMax, sizeof(s_f_Formula))))
		s_OOMError();

	/* garbage collection should be prepared in any case */
	
	s_f_freeFBHead = NewFormulaBlock();
	SetBaseType(s_f_freeFBHead);
	s_f_freeFBTail = s_f_freeFBHead;
}

int ReadFormulaBlock(s_f_FB** retFBP, int *retDivDepth)
{
int termDivDepth;
s_f_FB *curSumBlock, *sumBlock;
int err;
int divDepth = 0;

	SkipSpacesAndReadCont();
	
	/* first term */	
	sumBlock = NewFormulaBlock();
	sumBlock->operand = ReadTopSign();
	ErrTest(ReadTerm(&sumBlock,&termDivDepth));

	divDepth = termDivDepth;

	SkipSpacesAndReadCont();

	if (s_f_str[0] == '\0')		/* if only one term then return */
	{

		if (sumBlock->type == BaseType)
		{
			*retFBP = sumBlock;
		}
		else if (sumBlock->operand == AddOp)/*BlockType*/
		{
			*retFBP = sumBlock->block;
			free(sumBlock);
		}
		else
		{
			*retFBP = sumBlock;
		}
		*retDivDepth = divDepth;
		return 0;
	}
	
	/* at least two terms */
	
	curSumBlock = sumBlock;
	
	while( 1 )
	{

		SkipSpacesAndReadCont();

		switch(s_f_str[0])
		{
		case '\0':
		case ')':
#if TEST_CO
		case '}':
#endif
			goto end;
			break;
			
		case '+':
		case '-':
			curSumBlock = MakeNextBlock(curSumBlock);
			if (s_f_str[0] == '+')
				curSumBlock->operand = AddOp;
			else
				curSumBlock->operand = SubOp;
				
			SFStrInc();
			ErrTest(ReadTerm(&curSumBlock, &termDivDepth));
			divDepth = max(divDepth, termDivDepth);
			break;
		
		default :
			printf("In ReadFormulaBlock:\nIllegal character '%c'\n",s_f_str[0]);
			return s_ReadFBErr;
			break;
		}
		
	}

end:
	*retFBP = sumBlock;
	*retDivDepth = divDepth;
	return s_NoErr;
}

int ReadTopSign(void)
{
int topOp;

	SkipSpacesAndReadCont();

	if (s_f_str[0] == '-' )
	{
		SFStrInc();
		SkipSpacesAndReadCont();
		topOp = SubOp;
	}
	else if (s_f_str[0] == '+' )
	{
		SFStrInc();
		SkipSpacesAndReadCont();
		topOp = AddOp;
	}
	else
	{
		topOp = AddOp;
	}
	return topOp;
}

int ReadTerm(s_f_FB** retSumBlockP, int* retDivDepthP)
{
s_f_FB *prodBlock, *curBlock, *symbolFormulaBlock;
int divDepth = 0, curDivDepth = 0, numDepth = 0;
int divFlag = 0, allowCharFlag = 0;
int err;


	curBlock = prodBlock = NewFormulaBlock();

	while(1)
	{
		SkipSpacesAndReadCont();
		
		if (s_f_str[0] == '(' )
		{
			curBlock->type = BlockType;
			ErrTest(ReadParen(&(curBlock->block), &curDivDepth) );  /* change position or power ?? */
		}
		else /*should be symbols*/
		{
			SetBaseType(curBlock);
			ErrTest(ReadFormulaBase(curBlock->base,&curDivDepth, &symbolFormulaBlock));
			
			/* if symbol is already defined as formula, substitute it. */
			if (symbolFormulaBlock)
			{
				divDepth = max(divDepth,curDivDepth);
				
				curBlock->type = BlockType;
				curBlock->block = symbolFormulaBlock;
				curBlock->base = 0;
			}
		}
		
		SkipSpacesAndReadCont();

		switch(s_f_str[0])
		{
		case '\0':
		case ')':
		case '+':
		case '-':
#if TEST_CO
		case '}':
#endif
					/*end*/
			if (prodBlock->next)	/* more than two blocks */
			{
				prodBlock->operand = MulOp;
				retSumBlockP[0]->type = BlockType;
				retSumBlockP[0]->block = prodBlock;
			}
			else	/* one block */
			{
				prodBlock->operand = retSumBlockP[0]->operand;
				**retSumBlockP = *prodBlock;
			}
			
			if (divFlag) /* frac is in front */
				curDivDepth = max(numDepth,curDivDepth)+1;
			divDepth = max(divDepth,curDivDepth);

			*retDivDepthP = divDepth;
			return s_NoErr;
			break;
			
		case '/':
			curBlock = MakeNextBlock(curBlock);
			curBlock->operand = DivOp;
			
			curDivDepth = max(numDepth,curDivDepth)+divFlag;
			numDepth = curDivDepth;
			divFlag = 1;
			
			SFStrInc();
			break;
			
		case '*':
			SFStrInc();
			SkipSpacesAndReadCont();

			/*continue to next*/				
		case '(':
			allowCharFlag = 1;

			/*continue to next*/				
		default:
			if (!( isalnum(s_f_str[0]) || s_f_str[0]=='\\' || allowCharFlag ))
			{
				printf("In ReadTerm:\nIllegal character '%c'\n",s_f_str[0]);
				return s_ReadFBErr;
			}
			allowCharFlag = 0;
			
			/*is multplication*/
			curBlock = MakeNextBlock(curBlock);
			curBlock->operand = MulOp;
			
			if (divFlag) /* frac is in front */
				curDivDepth = max(numDepth,curDivDepth) + 1;
			divDepth = max(divDepth,curDivDepth);
			numDepth = 0;
			divFlag = 0;

			break;
		}
	}
}

int ReadParen(s_f_FB** retBlockP, int* retDivDepthP)
{
int err;

	SFStrInc();
	
	ErrTest(ReadFormulaBlock(retBlockP, retDivDepthP) );

	SkipSpacesAndReadCont();


	if (s_f_str[0] == ')' )
	{
		SFStrInc();
		SkipSpacesAndReadCont();
		if (s_f_str[0] == '^' )
		{
			SFStrInc();
			retBlockP[0]->power = ReadCoef();
		}
	}
	else
	{
		printf("In ReadParen:\nIllegal character '%c'\n",s_f_str[0]);
		return s_ReadFBErr;
	}
	
	return s_NoErr;
}


int ReadFormulaBase(s_f_Base* base, int *depth, s_f_FB **retBlockP)
{
int i;
int bsign = 1;
s_f_FB *retBlock = 0, *curBlock = 0;
#if !TEST_CO
int nameUsedAsFormulaOrOperatorFlag = 0;
#endif
int err;

#if TEST_CO

s_f_FormulaPB formulaPB;
s_f_ControlPB controlPB;

int curDepth;

#else /*TEST_CO*/

int readAlphasFN, readAlphasPower;
int controlNumber, controlType;

#endif /*else TEST_CO*/


#if TEST_CO

	formulaPB.usedFlag = 0;
	controlPB.usedFlag = 0;
	controlPB.operatorFlag = 0;

#endif /*TEST_CO*/

	SkipSpacesAndReadCont();
	
	if (s_f_str[0] == '-')
	{
		bsign = -1;
		SFStrInc();
		SkipSpacesAndReadCont();		
	}
	else
	{
		bsign = 1;
	}

	if ( isdigit(s_f_str[0]) )
	{
		base->coef = bsign * ReadCoef();
	}
	else if (s_f_str[0] == '.')
	{
		SFStrInc();

		ReadContIfNeed();
		
		if (isdigit(s_f_str[0]))
		{
			base->coef = bsign * ReadDecimalCoef();
		}
		else
		{
			PrintError("In ReadFormulaBase:\nIllegal character '%c'",s_f_str[0]);
		}
	}
	else
	{
		base->coef = bsign;
	}
	
	SkipSpacesAndReadCont();
	
	for(i=0;i<s_f_sNum;i++)
		base->power[i] = 0;

	while( isalpha (s_f_str[0]) || s_f_str[0] == '\\' )
	{
#if TEST_CO
		ErrTest(ReadAlphas(base,&formulaPB,&controlPB));

		if (formulaPB.usedFlag)
		{
			if(!retBlock)
			{
				retBlock = NewFormulaBlock();
				retBlock->type = BaseType;
				retBlock->operand = MulOp;
				retBlock->base = base;
				curBlock = retBlock;
			}
			curBlock = MakeNextBlock(curBlock);
			curBlock->type = BlockType;
			curBlock->operand = MulOp;
			*depth = s_f_formulas[formulaPB.num].divDepth;
			if (formulaPB.power == 1)
			{
				curBlock->block = CopyFB(s_f_formulas[formulaPB.num].fb);
			}
			else
			{
				curBlock->block = NewFormulaBlock();
				//curBlock->block->top = 1; /* old code */
				curBlock->block->power = formulaPB.power;
				curBlock->block->type = BlockType;
				curBlock->block->block = CopyFB(s_f_formulas[formulaPB.num].fb);
			}
		}

		if (controlPB.operatorFlag) /* control operator */
		{
			if(!retBlock)
			{
				retBlock = NewFormulaBlock();
				retBlock->type = BaseType;
				retBlock->operand = MulOp;
				retBlock->base = base;
				curBlock = retBlock;
			}
			curBlock = MakeNextBlock(curBlock);
			curBlock->type = BlockType;
			curBlock->operand = MulOp;
			
			ErrTest(ReadControlOperator(controlPB, &(curBlock->block), &curDepth));
			
			*depth = max(*depth, curDepth);
		}

		SkipSpacesAndReadCont();

#else /*TEST_CO*/

		ErrTest(ReadAlphas(base,&nameUsedAsFormulaOrOperatorFlag, 
					&readAlphasFN, &readAlphasPower, &controlType, &controlNumber));
		if (nameUsedAsFormulaOrOperatorFlag)
		{
			if(!retBlock)
			{
				retBlock = NewFormulaBlock();
				retBlock->type = BaseType;
				retBlock->operand = MulOp;
				retBlock->base = base;
				curBlock = retBlock;
			}
			curBlock = MakeNextBlock(curBlock);
			curBlock->type = BlockType;
			curBlock->operand = MulOp;
			*depth = s_f_formulas[readAlphasFN].divDepth;
			if (readAlphasPower == 1)
			{
				curBlock->block = CopyFB(s_f_formulas[readAlphasFN].fb);
			}
			else
			{
				curBlock->block = NewFormulaBlock();
				//curBlock->block->top = 1; /* old code */
				curBlock->block->power = readAlphasPower;
				curBlock->block->type = BlockType;
				curBlock->block->block = CopyFB(s_f_formulas[readAlphasFN].fb);
			}
		}
		SkipSpacesAndReadCont();

#endif /*else TEST_CO*/
	}
	
	*retBlockP = retBlock;
	return s_NoErr;
}

double ReadCoef(void)
{
double num = 0;

	ReadContIfNeed();
	
	while (isdigit(s_f_str[0]))
	{
		num = num*10 + s_f_str[0] - '0';
		SFStrInc();
	}
	if (s_f_str[0] == '.')
	{
		SFStrInc();
		num += ReadDecimalCoef();
	}
	return num;
}

double ReadDecimalCoef()
{
double num = 0;
double denom = 1;

	ReadContIfNeed();

	while (isdigit(s_f_str[0]))
	{
		num = num*10 + s_f_str[0] - '0';
		denom *= 10;
		SFStrInc();
	}
	return num/denom;
}

#if TEST_CO
int ReadAlphas(s_f_Base* base, s_f_FormulaPB *formulaPBP, s_f_ControlPB *controlPBP)
{
#else
int ReadAlphas(s_f_Base* base, int *symbolFormulaNameFlagP, int *readAlphasFNP,
					int *readAlphasPowerP, int *controlTypeP, int *controlNumberP)
{
int controlType;
int controlFlag = 0;
int controlNum = 0;
#endif
char symbolName[S_F_MAX_NAME_LENGTH+1];
char printName[2*S_F_MAX_NAME_LENGTH+1] = "";
int sn;
int err;

	SkipSpacesAndReadCont();

	if (!IsNameStart(s_f_str[0]))
	{
		return 0;
	}
	else
	{
#if TEST_CO
		ErrTest(ReadControlOrSymbolName(symbolName, printName, controlPBP));
#else
		ErrTest(ReadControlOrSymbolName(symbolName, printName, controlNumberP, &controlFlag));
#endif
	}

#if TEST_CO

	if (controlPBP->usedFlag && (controlPBP->type == operatorType))
	{
		controlPBP->operatorFlag = 1;
		controlPBP->operatorNum = controlTable[controlPBP->num].operatorNum;
		return s_NoErr;
	}

#endif

#if TEST_CO

	formulaPBP->num = FormulaNumber(symbolName);
	if (formulaPBP->num < s_f_fNum) /* symbol is used as formula name */
	{
		if (s_f_str[0] == '^')
		{
			SFStrInc();
			formulaPBP->power = ReadCoef();
		}
		else
		{
			formulaPBP->power = 1;
		}
		
		formulaPBP->usedFlag = 1;		
		return s_NoErr;;
	}
	else
	{
		formulaPBP->usedFlag = 0;
	}

	sn = SymbolNumber(symbolName);
	if ( sn >= s_f_sNum)
	{
		if (controlPBP->usedFlag)
			AddSymbol(symbolName, controlPBP->usedFlag, controlPBP->num, printName);
		else
			AddSymbol(symbolName, controlPBP->usedFlag, controlPBP->num, 0);
	}

#else /*TEST_CO*/

	*readAlphasFNP = FormulaNumber(symbolName);
	if (*readAlphasFNP < s_f_fNum) /* symbol is used as formula name */
	{
		if (s_f_str[0] == '^')
		{
			SFStrInc();
			*readAlphasPowerP = ReadCoef();
		}
		else
		{
			*readAlphasPowerP = 1;
		}
		
		*symbolFormulaNameFlagP = 1;		
		return s_NoErr;;
	}
	else
	{
		*symbolFormulaNameFlagP = 0;
	}

	sn = SymbolNumber(symbolName);
	if ( sn >= s_f_sNum)
	{
		if (controlFlag)
			AddSymbol(symbolName, controlFlag, controlNum, printName);
		else
			AddSymbol(symbolName, controlFlag, controlNum, 0);
	}

#endif /*else TEST_CO*/
	
	if (s_f_str[0] == '^')
	{
		SFStrInc();
		base->power[sn] = ReadCoef();
	}
	else
	{
		base->power[sn] = 1;
	}

	return s_NoErr;

#if TEST_CO
}
#else
}
#endif

int ReadSymbolName(char *sName)
{
char *curSName;
int ret;

	curSName = sName;
	while(isalnum(*s_f_str)||(*s_f_str=='_'))
	{
		if (*s_f_str=='_')
		{
			ret = ReadSubscript(&curSName);
			*curSName = '\0';
			return ret;
		}
		SFCopyIncChar(&curSName);
	}
	*curSName = '\0';
	return s_NoErr;
}

int ReadSubscript(char **curSNameP)
{
	SFCopyIncChar(curSNameP);
	if (*s_f_str=='{')
	{
		SFCopyIncChar(curSNameP);
		
		while(isalnum(*s_f_str)||(*s_f_str=='_'))
		{
			if (*s_f_str=='_')
			{
				return ReadSubscript(curSNameP);
			}
			SFCopyIncChar(curSNameP);
		}
		if ( *s_f_str != '}' )
		{
			s_printf("In ReadSubscript:\nIllegal character '%c'\n", *s_f_str);
			return s_ReadFBErr; /* error */
		}
	}
	SFCopyIncChar(curSNameP);
	return s_NoErr; /* no error */	
}

void SFCopyIncChar(char **curSNameP)
{
	**curSNameP = *s_f_str;
	SFStrInc();
	(*curSNameP)++;
}


int AddFormula(char *formulaName, int *fnP, char *printFName, int controlFlag)
{
int fn;

	if ((fn = FormulaNumber(formulaName))>=s_f_fNum)
	{
		if (fn>=s_f_formulaMax)
		{
			printf("Error: Too many Formulas.");
			return -2;
		}
		else
		{
			if (!(s_f_fName[fn] = calloc(strlen(formulaName),sizeof(char))))
				s_OOMError();
			strcpy(s_f_fName[fn],formulaName);
			s_f_formulas[fn].fb = NewFormulaBlock();
			s_f_fNum++;
		}
	}
	s_f_formulas[fn].divDepth = 0;
	s_f_formulas[fn].expanded = 0;
	s_f_formulas[fn].sorted = 0;
	s_f_formulas[fn].frac = 0;
	s_f_formulas[fn].cleanComplexFlag = 0;
	
	s_f_formulas[fn].controlFlag = controlFlag;
	if (controlFlag)
	{
		if (!(s_f_formulas[fn].printName = calloc(strlen(printFName), sizeof(char*))))
			s_OOMError();
		strcpy(s_f_formulas[fn].printName,printFName);
	}
	else
		s_f_formulas[fn].printName = s_f_fName[fn];
	*fnP = fn;
	return s_NoErr;
}

void AddSymbol(char *c, int controlFlag, int controlNum, char *printName)
{

	if (s_f_sNum>=s_f_symbolMax)
	{
		s_printf("Too many symbols.  Ignored.\n");
	}
	else
	{
		if (!(s_f_sName[s_f_sNum] = calloc(strlen(c)+1,sizeof(char))))
			s_OOMError();
		strcpy(s_f_sName[s_f_sNum],c);
		s_f_symbolTable[s_f_sNum].name = s_f_sName[s_f_sNum];
		if (printName)
		{
			if (!(s_f_symbolTable[s_f_sNum].printName = calloc(strlen(printName)+1, sizeof(char*))))
				s_OOMError();
			strcpy(s_f_symbolTable[s_f_sNum].printName,printName);
		}
		else
			s_f_symbolTable[s_f_sNum].printName = s_f_symbolTable[s_f_sNum].name;
		s_f_symbolTable[s_f_sNum].controlFlag = controlFlag;
		s_f_symbolTable[s_f_sNum].controlNum = controlNum;
		s_f_sNum++;
	}

}


/*-------------------- Control Sequence --------------------*/

#if TEST_CO

int ReadControlOrSymbolName(char *symbolName, char *printName, s_f_ControlPB *controlPBP)
{
char controlName[S_F_MAX_NAME_LENGTH+1];
int err;

	if (s_f_str[0] == '\\') /* control sequence */
	{
		SFStrInc();
		ErrTest(ReadControlName(controlName));
		ErrTest(ControlSequence(controlName, &(controlPBP->num), &(controlPBP->type), symbolName));

		controlPBP->usedFlag = 1;

		ErrTest(MakeCSSymbolName(controlPBP->num, controlName, symbolName, printName));
	}
	else /* alphabet */
	{
		ErrTest(ReadSymbolName(symbolName));
		controlPBP->usedFlag = 0;
	}
	return s_NoErr;
}

#else /*TEST_CO*/

int ReadControlOrSymbolName(char *symbolName, char *printName, int *controlNumP, int *controlFlagP)
{
int controlType;
char controlName[S_F_MAX_NAME_LENGTH+1];
int err;

	if (s_f_str[0] == '\\') /* control sequence */
	{
		SFStrInc();
		ErrTest(ReadControlName(controlName));
		ErrTest(ControlSequence(controlName, controlNumP, &controlType, symbolName));
		*controlFlagP = 1;
		ErrTest(MakeCSSymbolName(*controlNumP, controlName, symbolName, printName));
	}
	else /* alphabet */
	{
		ErrTest(ReadSymbolName(symbolName));
		*controlFlagP = 0;
	}
	return s_NoErr;
}

#endif /*else TEST_CO*/

int ReadControlName(char *sName)
{
char *curSName;

	curSName = sName;
	while(isalnum(*s_f_str))
	{
		SFCopyIncChar(&curSName);
	}
	*curSName = '\0';
	return s_NoErr;
}

int ControlSequence(char *controlName, int *controlNumP, int *controlTypeP, char *symbolName)
{
int controlNumber;
int controlType;

	*controlNumP = controlNumber = FindControlNumber(controlName);
	if (!controlTable[controlNumber].controlName)
	{
		s_printf("In ControlSequence:\nControl Sequence \\%s is not defined.\n", controlName);
		return s_ReadFBErr;
	}

	*controlTypeP = controlType = controlTable[controlNumber].type;
	
	switch(controlType)
	{
	case symbolType:
		symbolName[0] = '\\';
		strcpy(symbolName+1, controlName);
		break;
	default:
		;
	}
	return s_NoErr;
}

int FindControlNumber(char *controlName)
{
int cn = 0;

	for ( cn = 0;
			controlTable[cn].controlName /* 0 at the end */
			&& strcmp(controlName, controlTable[cn].controlName); 
			cn++);

	return cn;
}

int MakeCSSymbolName(int controlNum, char *controlName, char *symbolName, char *printName)
{
char subscript[S_F_MAX_NAME_LENGTH+1], *curSubscript;
int err;

	curSubscript = subscript;
	sprintf(symbolName, "\\%s", controlName);
	strcpy(printName, controlTable[controlNum].printName);

	if (*s_f_str == '_')
	{
		ErrTest(ReadSubscript(&curSubscript) );
		*curSubscript = '\0';
		strcat(symbolName, subscript);
		strcat(printName, subscript);
	}

	return s_NoErr;
}

#if TEST_CO
int ReadControlOperator(s_f_ControlPB controlPB, s_f_FB **retBlockP, int *retDepthP)
{
int err;
int fracDepth = 0;



	if (controlPB.operatorNum == fracOp)
	{
		ErrTest(ReadControlFraction(retBlockP, &fracDepth));

		*retDepthP = fracDepth+1;		
		
		return s_NoErr;
	}
	else
	{
		return -1;
	}
}

int ReadControlFraction(s_f_FB **retBlockP, int *retDepthP)
{
s_f_FB *retBlock, *curFB;
int numDepth, denomDepth;
int err;

	SkipSpacesAndReadCont();	
	if (s_f_str[0] != '{')
		goto charError;	
	SFStrInc();
	
	retBlock = NewFormulaBlock();
	retBlock->operand = MulOp;

	ErrTest(ReadFormulaBlock(&(retBlock->block), &numDepth));
	
	SkipSpacesAndReadCont();	
	if (s_f_str[0] != '}')
		goto charError;	
	SFStrInc();

	SkipSpacesAndReadCont();	
	if (s_f_str[0] != '{')
		goto charError;	
	SFStrInc();

	curFB = MakeNextBlock(retBlock);
	curFB->operand = DivOp;

	ErrTest(ReadFormulaBlock(&(curFB->block), &denomDepth));
	
	SkipSpacesAndReadCont();	
	if (s_f_str[0] != '}')
		goto charError;	
	SFStrInc();

	*retBlockP = retBlock;
	*retDepthP = max(numDepth, denomDepth);
	return s_NoErr;

charError:
	printf("In ReadControlFraction:\nIllegal Character %c\n", s_f_str[0]);
	return -1;
}
#endif /*TEST_CO*/


/******************************* Print Formula ********************************/

int s_f_printTimeFlag = 0;

void BeginFormulaPrinting(void)
{
unsigned long sec;
DateForm shortDate = 0;
Str255 dateString;
Boolean noSec = 0;
Str255 timeString;

	s_f_printFlag = 1;
	s_f_printContFlag = 0;
	BeginPrinting();

	if (s_f_printTimeFlag)
	{
		GetDateTime(&sec);
		DateString(sec, shortDate, dateString, 0); /*not sure if it works on classic */
		TimeString(sec, noSec, timeString, 0);

		ScrollFormula(1);
		MoveTo(0,s_f_fy);
		TextSize(s_f_fontSize-2);
		DrawString(dateString);
#if !S_UNIX
		DrawString("\p   ");
#else
		DrawString("   ");
#endif
		DrawString(timeString);
		TextSize(s_f_fontSize);
		ScrollFormula(1);
	}
}

void EndFormulaPrinting(void)
{
	if (s_f_printFlag)
	{
		s_f_printFlag = 0;
		if (s_f_printContFlag)
		{
			EndPrintingBlind();
		}
		else
			EndPrinting();
	}
}

/**************************** Change Input Device *****************************/

void SetFormulaInputFile(void)
{
char *filename;

	filename = s_f_str;

#if !S_UNIX
	if (DDCount())
		s_f_input = DDFOpen("Input File", "r");
#endif /*!S_UNIX*/
	else if (filename&&filename[0])
	{
			SkipSpaces(&filename);
			s_f_input = fopen(filename,"r");
	}
	else
		s_f_input = DialogFOpen("Input File", "r");
//s_printf("s_f_input = %p\n", s_f_input);		
	if (!s_f_input)
		s_f_input = stdin;
}

void SetFormulaInputConsole(void)
{
	s_f_input = stdin;
}


/**************************** SetInputShowFlag *****************************/

void SetInputShowFlag(void)
{
	SetFlag(&s_f_inputShowFlag);
}

#endif /* pragma once */
