/*
 *  Formula-IO.h
 *  Version 1.8.1 / 000830
 *  by Satoshi Takahashi
 */


#ifndef FORMULA_IO_H  /*pragma once*/
#define FORMULA_IO_H

/*********************************** Switches **********************************/

#include <Formula-Switches.h>

/*********************************** Headers **********************************/

#include <Formula-v.h>


/****************************** Input Command Line *****************************/

int ReadOneLine(void);


/*************************** Output Formula ***********************************/

extern int s_f_showFlag;

void PrintFormula(void);
void PrintNumFormula(int n);
void PrintFormulaBlock(s_f_FormulaBlock *blockP, int divLevel);

void PrintOperand(s_f_FB *blockP, int topFlag, int *vDivFlagP,
						Point *divRPtP, Point divLPt);
void VDivUpper(int *vDivFlagP, int *divLevelP, Point *divLPtP);
void VDivLower(int *vDivFlagP, Point divLPt, Point *divRPtP);
void VDivEnd(int *vDivFlagP, int *divLevelP, Point divRPt, Point divLPt);
int  CheckPrintParen(s_f_FB *blockP, int vDivFlag);
void PrintPower(int power);

void PrintFormulaBase(s_f_Base *base);
void ScrollFormula(int n);
void CheckRightMargin(void);
void ClearFormulaDisp(void);
/*-------------------- Rewrite --------------------*/
int CheckRightBound(void);
void EraseFormulaBlock(Point prevPenLoc);

/**************************** Save Formula ***********************************/

void BeginFormulaSaving(void);
void EndFormulaSaving(void);
void DrawFormulaCString(char *str);


/******************************* Print Formula ********************************/

extern int s_f_printTimeFlag;

void BeginFormulaPrinting(void);
void EndFormulaPrinting(void);


/**************************** Input Formula ***********************************/

void SetFormulaByString(void);
void InitilaSetSetUp(void);
int ReadFormulaBlock(s_f_FB** returnFBP, int *retDivDepth);
int ReadTopSign(void);
int ReadTerm(s_f_FB** retSumBlockP, int* retDivDepthP);
int ReadParen(s_f_FB** retBlockP, int* retDivDepthP);
int ReadFormulaBase(s_f_Base* base, int *depth, s_f_FB **retBlockP);
double ReadCoef(void);
double ReadDecimalCoef(void);

#if TEST_CO
int ReadAlphas(s_f_Base* base, s_f_FormulaPB *formulaPBP, s_f_ControlPB *controlPBP);
#else
int ReadAlphas(s_f_Base* base, int *symbolFormulaNameFlagP,
					int *readAlphasFN, int *readAlphasPower,
						int *controlNumberP, int *controlType);
#endif

int ReadSymbolName(char *sName);
int ReadSubscript(char **curSNameP);
void SFCopyIncChar(char **curSNameP);

int AddFormula(char *formulaName, int *fnP,
				char *printFName, int controlFlag);
void AddSymbol(char *c, int controlFlag, int controlNum, char *printName);

/*-------------------- Control Sequence --------------------*/

#if TEST_CO
int ReadControlOrSymbolName(char *symbolName,
			char *printName, s_f_ControlPB *controlPBP);
#else
int ReadControlOrSymbolName(char *symbolName, char *printName,
								int *controlNum, int *controlFlagP);
#endif
int ReadControlName(char *sName);
int ControlSequence(char *controlName, int *controlNumP,
					int *controlTypeP, char *symbolName);
int FindControlNumber(char* controlName);
int MakeCSSymbolName(int controlNum, char *controlName,
						char *symbolName, char *printName);
#if TEST_CO
int ReadControlOperator(s_f_ControlPB controlPB, s_f_FB **retBlockP, int *retDepthP);
int ReadControlFraction(s_f_FB **retBlockP, int *retDepth);
#endif

/**************************** Change Input Device *****************************/

void SetFormulaInputFile(void);
void SetFormulaInputConsole(void);


/**************************** SetInputShowFlag *****************************/

void SetInputShowFlag(void);

#endif /*pragma once*/
