/*
 *  GraphCommon.c
 *  Version 1.9.1 / 010219
 *  by Satoshi Takahashi
 */

#ifndef GRAPH_COMMON_C
#define GRAPH_COMMON_C

#include <string.h>
#include <math.h>

#include <PDProcs.h>
#include <Plot.h>
#include <s_Drawing.h>
#include <StartSCL.h>
#include <ParametersDialog.h>
#include <s_RedrawFunc.h>
#include <s_Common.h>
#include <s_Math.h>
#include <EPS.h>

#include <Graph1D.h>
#include <GraphSpreadSheet.h>

#include <GraphCommon.h>
#include <VarBody.h>
#include <GraphCommon-v.c>



void GraphAddParameters1(void)
{
	AddParameterP(&SavePDParameters, doType, "Save Params", s_g.dialogP);
	AddParameterP(s_PD.paramFileName, stringType, "Param File Name", s_g.dialogP);
	AddParameterP(&LoadPDParameters, doType, "Load Params", s_g.dialogP);
	AddParameterP(&s_g.exitFlag, flagType, "Exit Graph", s_g.dialogP);
	AddParameterP(&s_g.saveFlag, flagType, "Save Graph", s_g.dialogP);
	AddParameterP(&s_g.epsSaveFlag, flagType, "EPS Save Graph", s_g.dialogP);
	AddParameterP(&s_g.printFlag, flagType, "Print Graph", s_g.dialogP);
	AddParameterP(&SaveProc, doType, "Bitmap Save", s_g.dialogP);
	AddParameterP(&GraphEPSSaveProc, doType, "EPS Bitmap Save", s_g.dialogP);
	AddParameterP(&PrintProc, doType, "Bitmap Print", s_g.dialogP);
}

void GraphFrameAddParameters(void)
{
	AddParameterP(&s_g.limXMin, doubleType, "Min Frame X", s_g.dialogP);
	AddParameterP(&s_g.limXMax, doubleType, "Max Frame X", s_g.dialogP);
	AddParameterP(&s_g.limYMin, doubleType, "Min Frame Y", s_g.dialogP);
	AddParameterP(&s_g.limYMax, doubleType, "Max Frame Y", s_g.dialogP);
	AddParameterP(&s_g.showXMin, flagType, "Show X Min", s_g.dialogP);
	AddParameterP(&s_g.showXMax, flagType, "Show X Max", s_g.dialogP);
	AddParameterP(&s_g.showYMin, flagType, "Show Y Min", s_g.dialogP);
	AddParameterP(&s_g.showYMax, flagType, "Show Y Max", s_g.dialogP);
	AddParameterP(&s_g.left, intType, "Left Position", s_g.dialogP);
	AddParameterP(&s_g.bottom, intType, "Bottom Position", s_g.dialogP);
	AddParameterP(&(s_g.width), intType, "Width", s_g.dialogP);
	AddParameterP(&(s_g.height), intType, "Height", s_g.dialogP);
	AddParameterP(&s_g.frameWidth, doubleType, "Frame Width", s_g.dialogP);
	AddParameterP(&s_g.frame, flagType, "Draw Frame", s_g.dialogP);
	AddParameterP(&s_g.xAxis, flagType, "Draw X Axis", s_g.dialogP);
	AddParameterP(&s_g.yAxis, flagType, "Draw Y Axis", s_g.dialogP);
	AddParameterP(&s_g.arrow, flagType, "Draw Arrow", s_g.dialogP);
	AddParameterP(&s_g.xLabel, stringType, "X Label", s_g.dialogP);
	AddParameterP(&s_g.yLabel, stringType, "Y Label", s_g.dialogP);
	AddParameterP(&s_g.labelSize, shortType, "Label Size", s_g.dialogP);
	AddParameterP(&s_g.xLabelPositionX, intType, "X Label X Pos.", s_g.dialogP);
	AddParameterP(&s_g.xLabelPositionY, intType, "X Label Y Pos.", s_g.dialogP);
	AddParameterP(&s_g.yLabelPositionX, intType, "Y Label X Pos.", s_g.dialogP);
	AddParameterP(&s_g.yLabelPositionY, intType, "Y Label Y Pos.", s_g.dialogP);
	AddParameterP(&s_g.xMinLabelDigit, intType, "X Min digit", s_g.dialogP);
	AddParameterP(&s_g.xMaxLabelDigit, intType, "X Max digit", s_g.dialogP);
	AddParameterP(&s_g.yMinLabelDigit, intType, "Y Min digit", s_g.dialogP);
	AddParameterP(&s_g.yMaxLabelDigit, intType, "Y Max digit", s_g.dialogP);
	AddParameterP(&s_p.xNumPositionX, intType, "X Num X Pos.", s_g.dialogP);
	AddParameterP(&s_p.xNumPositionY, intType, "X Num Y Pos.", s_g.dialogP);
	AddParameterP(&s_p.yNumPositionX, intType, "Y Num X Pos.", s_g.dialogP);
	AddParameterP(&s_p.yNumPositionY, intType, "Y Num Y Pos.", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkDrawX, flagType, "Scale Mark X", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkIntervalX, doubleType, "Interval", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkShowNumX, flagType, "Show Number", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkNumXDigit, intType, "Digit", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkDrawY, flagType, "Scale Mark Y", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkIntervalY, doubleType, "Interval", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkShowNumY, flagType, "Show Number", s_g.dialogP);
	AddParameterP(&s_g.scaleMarkNumYDigit, intType, "Digit", s_g.dialogP);
/* Title */
	AddParameterP(&s_g.graphTitle, stringType, "Title", s_g.dialogP);
	AddParameterP(&s_g.upperTitelFlag, flagType, "Title above", s_g.dialogP);
	AddParameterP(&s_g.lowerTitelFlag, flagType, "Title below", s_g.dialogP);
	AddParameterP(&s_g.titleX, intType, "Title X Position", s_g.dialogP);
	AddParameterP(&s_g.titleY, intType, "Title Y Position", s_g.dialogP);
}

void GraphFrameAndLabel(void)
{
double xMin, xMax, yMin, yMax;
char scale_num[30];
int aLWidth = 0;
short txSize;

	txSize = GetPortTextSize(s_thePort);
	SetFrame2DXMinDigit(s_g.xMinLabelDigit);
	SetFrame2DXMaxDigit(s_g.xMaxLabelDigit);
	SetFrame2DYMinDigit(s_g.yMinLabelDigit);
	SetFrame2DYMaxDigit(s_g.yMaxLabelDigit);

	s_PenSize(s_g.frameWidth/s_d_scale,s_g.frameWidth/s_d_scale);
	SetFont(s_g.defaultFont);
	s_TextSize(s_g.labelSize/s_d_scale);
	
	
	if (s_g.dim == 1 && s_g1.type == fType && s_fg.barFlag)
	{
		xMin = GraphConvertX(s_g.limXMin - 0.5*s_fg.dx);
		xMax = GraphConvertX(s_g.limXMax + 0.5*s_fg.dx);
	}
	else
	{
		xMin = GraphConvertX(s_g.limXMin);
		xMax = GraphConvertX(s_g.limXMax);
	}
	yMin = GraphConvertY(s_g.limYMin);
	yMax = GraphConvertY(s_g.limYMax);
	
	if (s_g.frame)
	{
		s_p.showXMin = s_g.showXMin;
		s_p.showXMax = s_g.showXMax;
		s_p.showYMin = s_g.showYMin;
		s_p.showYMax = s_g.showYMax;
		if ((s_g1.type == fType) && (s_fg.barFlag))
			PlotFrame2DSeparate(xMin, xMax, yMin, yMax,
						GraphConvertX(s_g.limXMin), GraphConvertX(s_g.limXMax),
						yMin, yMax);
		else
			PlotFrame2D(xMin, xMax, yMin, yMax);
	}
	if (s_g.xAxis)
	{
		Plot2DMoveTo(xMin, 0);
		Plot2DLineTo(xMax, 0);
		if (!s_g.frame)
		{
			if (s_g.arrow)
				AxisArrow(Plot2DTransX(xMin), Plot2DTransY(0),
							Plot2DTransX(xMax), Plot2DTransY(0));
				
			if (s_g.showXMin)
			{
				sprintf(scale_num,"%.*f",s_p.frameXMinDigit, xMin);	
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Plot2DMoveTo(xMin,0);
				Move(((-aLWidth/2.0)+s_p.xNumPositionX)/s_d_scale,
						s_p.xNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
			}
			if (s_g.showXMax)
			{
				sprintf(scale_num,"%.*f",s_p.frameXMaxDigit,xMax);
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Plot2DMoveTo(xMax,0);
				Move(((-aLWidth/2.0)+s_p.xNumPositionX)/s_d_scale,
						s_p.xNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
			}
		}	
	}
	if (s_g.yAxis)
	{
		Plot2DMoveTo(0, yMin);
		Plot2DLineTo(0, yMax);
		if (!s_g.frame)
		{
			if (s_g.arrow)
				AxisArrow(Plot2DTransX(0), Plot2DTransY(yMin),
							Plot2DTransX(0), Plot2DTransY(yMax));

			if (s_g.showYMin)
			{
				sprintf(scale_num,"%.*f",s_p.frameYMinDigit,yMin);
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Plot2DMoveTo(0,yMin);
				Move((-aLWidth + s_p.yNumPositionX)/s_d_scale,
						s_p.yNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
			}
			if (s_g.showYMax)
			{
				sprintf(scale_num,"%.*f",s_p.frameYMaxDigit,yMax);
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Plot2DMoveTo(0,yMax);
				Move((-aLWidth + s_p.yNumPositionX)/s_d_scale,
						s_p.yNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
			}
		}
	}

	if (s_g.frame)
	{
		Plot2DMoveTo(xMax, yMin);
		Move(s_g.xLabelPositionX/s_d_scale, s_g.xLabelPositionY/s_d_scale);
		DrawGraphLabel(s_g.xLabel);
	}
	else if (s_g.xAxis)
	{
		Plot2DMoveTo(xMax, 0);
		Move(s_g.xLabelPositionX/s_d_scale, s_g.xLabelPositionY/s_d_scale);
		DrawGraphLabel(s_g.xLabel);
	}

	if (s_g.showXMax)
	{
		if (s_g.frame)
		{
			Plot2DMoveTo(xMin, yMax);
			Move(s_g.yLabelPositionX/s_d_scale, s_g.yLabelPositionY/s_d_scale);
			DrawGraphLabel(s_g.yLabel);
		}
		else if (s_g.yAxis)
		{
			Plot2DMoveTo(0, yMax);
			Move(s_g.yLabelPositionX/s_d_scale, s_g.yLabelPositionY/s_d_scale);
			DrawGraphLabel(s_g.yLabel);
		}
	}
    
    if (s_g.scaleMarkDrawX)
    {
    double x;
    
        s_PenPat(s_gray);
        for (x = xMin + s_g.scaleMarkIntervalX; x < xMax;
                x += s_g.scaleMarkIntervalX)
        {
            Plot2DMoveTo(x, yMax);
            Plot2DLineTo(x, yMin);
            if (s_g.scaleMarkShowNumX)
            {
				sprintf(scale_num,"%.*f",s_g.scaleMarkNumXDigit, x);
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Move((-aLWidth/2.0 + s_p.xNumPositionX)/s_d_scale,
						s_p.xNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
            }
        }
        s_PenPat(s_black);
    }
	
    if (s_g.scaleMarkDrawY)
    {
    double y;
    
        s_PenPat(s_gray);
        for (y = yMin + s_g.scaleMarkIntervalY; y < yMax;
                y += s_g.scaleMarkIntervalY)
        {
            Plot2DMoveTo(xMax, y);
            Plot2DLineTo(xMin, y);
            if (s_g.scaleMarkShowNumY)
            {
				sprintf(scale_num,"%.*f",s_g.scaleMarkNumYDigit, y);
				aLWidth = TextWidth(scale_num,0,strlen(scale_num));
				Move((-aLWidth + s_p.yNumPositionX)/s_d_scale,
						s_p.yNumPositionY/s_d_scale);
				DrawText(scale_num,0,strlen(scale_num));
            }
        }
        s_PenPat(s_black);
    }
	
	if (s_g.upperTitelFlag)
	{
		Plot2DMoveTo((xMax+xMin)/2,yMax);
		Move(0, -0.08*300/s_d_scale);
		Move(s_g.titleX,-s_g.titleY);
		CenterString(s_g.graphTitle);
		DrawStyleText("$&P");
		DrawGraphLabel(s_g.graphTitle);
		DrawStyleText("$&I");
	}
	if(s_g.lowerTitelFlag)
	{
		Plot2DMoveTo((xMax+xMin)/2,yMin);
		Move(0, 0.13*300/s_d_scale);
		Move(s_g.titleX,-s_g.titleY);
		CenterString(s_g.graphTitle);
		DrawStyleText("$&P");
		DrawGraphLabel(s_g.graphTitle);
		DrawStyleText("$&I");
	}

	SetFont(s_g.defaultFont);
	s_TextSize(txSize/s_d_scale);
}

void DrawGraphLabel(char *string)
{
	SetFont(s_g.defaultFont);
	s_TextSize(s_g.labelSize/s_d_scale);
	DrawStyleText(string);
}


void SetGraphXLabel(char* label)
{
	strcpy(s_g.xLabel,label);
}

void SetGraphYLabel(char* label)
{
	strcpy(s_g.yLabel,label);
}

void SetGraphTitle(char* title)
{
	strcpy(s_g.graphTitle,title);
}

void SetGraphUpperTitleFlag(int flag)
{
	s_g.upperTitelFlag=flag;
}

void SetGraphLowerTitleFlag(int flag)
{
	s_g.lowerTitelFlag=flag;
}

void SetGraphTitleXPosition(int position)
{
       s_g.titleX=position;
}

void SetGraphTitleYPosition(int position)
{
       s_g.titleY=position;
}

void SetGraphFrameWidth(double p)
{
	s_g.frameWidth = p;
}

void SetGraphLeftBottom(int left, int bottom)
{

	s_g.left = left;
	s_g.bottom = bottom;
}

void SetGraphWidthHeight(int width, int height)
{
	s_g.width = width;
	s_g.height = height;
}

void SetGraphFrameRange(double xMin, double xMax, double yMin, double yMax)
{
	s_g.limXMin = xMin;
	s_g.limXMax = xMax;
	s_g.limYMin = yMin;
	s_g.limYMax = yMax;
}


void AxisArrow(double ax1, double ay1, double ax2, double ay2)
{
short ax3, ay3, ax4, ay4, ax5, ay5;
double length;
double c, s;
PolyHandle poly;

	length = sqrt((ax1-ax2)*(ax1-ax2) + (ay1-ay2)*(ay1-ay2));
	ax3 = 10 * (ax1 - ax2) / length;
	ay3 = 10 * (ay1 - ay2) / length;
	
	c = cos(Pi/6.0);
	s = sin(Pi/6.0);
	
	ax4 = c*ax3 - s*ay3;
	ay4 = s*ax3 + c*ay3;
	
	ax5 = c*ax3 + s*ay3;
	ay5 = - s*ax3 + c*ay3;
	
	//ax2 -= 0.2 * ax3;
	//ay2 -= 0.2 * ay3;

	poly = OpenPoly();
	MoveTo(ax2, ay2);
	LineTo(ax2 + ax4, ay2 + ay4);
	LineTo(ax2 + ax5, ay2 + ay5);
	LineTo(ax2, ay2);
	ClosePoly();
	PaintPoly(poly);
	KillPoly(poly);
}


void GraphEPSSaveProc(void)
{
    switch(s_g.dim)
    {
    case 1:
        
        EPSBitMapSave(1, s_g.colorFlag, 0);
        break;
    case 2:
        EPSBitMapSave(1, 0, 0.5);
        break;
    case 3:
        EPSBitMapSave(8, 1, 0);
        break;
    }
}

#endif /*pragma once*/
