/*
 *  Newton.c
 *  Version 2.0.3 / 020723
 *  by Satoshi Takahashi & Maiko Watanabe
 */


#ifndef NEWTON_C /*pragma once*/
#define NEWTON_C
#include <s_Config.h>

#include <math.h>

#include <HugeArray.h>
#include <LinearSolve.h>

#include <Newton.h>
#include <VarBody.h>
#include <Newton-v.c>


int Newton1(double (*fp)(double), double (*dfp)(double), double *rootp)
{
int i = 0;
double prev;
double next;
double prevFVal;
double nextFVal;
double rate;
double change;

	next = newton.init;
	
    do
	{
		i++;
		if (i > NEWTON_LOOP)
			return -1; /* not solved */
		
		prev = next;
//s_printf("prev=%f\n", prev);
        prevFVal = (*fp)(prev);
        change = prevFVal/(*dfp)(prev);
//s_printf("f = %f, df = %f\n", prevFVal, (*dfp)(prev));
        rate = 1;
        do
        {
            next = prev - rate*change;
            nextFVal = (*fp)(next);
            rate /= 2;
        }while (nextFVal > prevFVal);
        prevFVal = nextFVal;
	}while (fabs(next - prev) > newton.precision);
	
	*rootp = next;
	return 0; /*solved*/
}

void SetNewton1Init(double init)
{
	newton.init = init;
}

void SetNewtonPrecision(double precision)
{
	newton.precision = precision;
}

void SetNewton1Precision(double precision)
{
	newton.precision = precision;
}

int Newton3(double (*fp)(double,double,double),double (*gp)(double,double,double),
            double (*hp)(double,double,double),double (*dfxp)(double,double,double),
            double (*dfyp)(double,double,double),double (*dfzp)(double,double,double),
            double (*dgxp)(double,double,double),double (*dgyp)(double,double,double),
			double (*dgzp)(double,double,double),double (*dhxp)(double,double,double),
            double (*dhyp)(double,double,double),double (*dhzp)(double,double,double),
			double *xp,double *yp,double *zp)
{
int i = 0;
double prevx,prevy,prevz;
double nextx,nexty,nextz;
double x[3];
double b[3];
double **A;

    A = MakeArrayDouble2(3,3);

    nextx = newton.initx;
    nexty = newton.inity;
    nextz = newton.initz;

	do
	{
		i++;
		if (i > NEWTON_LOOP)
		{
			FreeArrayDouble2(A, 3, 3);
			return -1;
		}

		prevx = nextx;
        prevy = nexty;
		prevz = nextz;
 
		b[0] = (*fp)(prevx,prevy,prevz);
        b[1] = (*gp)(prevx,prevy,prevz);
        b[2] = (*hp)(prevx,prevy,prevz);

		A[0][0] = (*dfxp)(prevx,prevy,prevz);
        A[0][1] = (*dfyp)(prevx,prevy,prevz);
        A[0][2] = (*dfzp)(prevx,prevy,prevz);
		A[1][0] = (*dgxp)(prevx,prevy,prevz);
        A[1][1] = (*dgyp)(prevx,prevy,prevz);
        A[1][2] = (*dgzp)(prevx,prevy,prevz);
        A[2][0] = (*dhxp)(prevx,prevy,prevz);
        A[2][1] = (*dhyp)(prevx,prevy,prevz);
        A[2][2] = (*dhzp)(prevx,prevy,prevz);
	
		LinearSolve(3,A,x,b);
		
		nextx = prevx - x[0];
        nexty = prevy - x[1];
        nextz = prevz - x[2];
        
        if (nextx > 1000.0 || nexty > 1000.0 || nextz > 1000.0)
        	return -1;
	} while((fabs(nextx- prevx) > newton.precision)||
		(fabs(nexty- prevy) > newton.precision)||
		(fabs(nextz- prevz) > newton.precision));

	*xp = nextx;	
	*yp = nexty;
    *zp = nextz;

	FreeArrayDouble2(A, 3, 3);
		return 0;
}

void SetNewton3Init(double initx,double inity,double initz)
{
	 newton.initx = initx;
     newton.inity = inity;
     newton.initz = initz;
}

void SetNewton3Precision(double precision)
{
	newton.precision = precision;
}

#endif /*pragma once*/