/*
 *  ParametersDialog.h
 *  Version 2.0 / 010824
 *  by Satoshi Takahashi
 */

#ifndef PARAMETERSDIALOG_H /*pragma once*/
#define PARAMETERSDIALOG_H

#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif

#include <stdio.h>

#if S_UNIX /* as s_Toolbox.h is not in parts-main */
#include <s_Toolbox.h>
#endif

/******************** Types ********************/

#define intType					1
#define staticIntType			2
#define longType				4
#define shortType				8

#define doubleType				11
#define expType					12

#define stringType				21
#define staticStringType		22

#define flagType				31

#define doType					41

#if !S_UNIX || XDialog
#define MaxPNum 26
#else
#define MaxPNum 22
#endif


typedef struct
{
	Ptr p;
	int	type;
	char comstr[255];
	Str255 com;
	char valstr[255];
	Str255 valpstr;
	union
	{
		char	charV;
		int		intV;
		long	longV;
		short	shortV;
		float	floatV;
		double	doubleV;
		void (*func)(void);
		char *stringV;
	} val;
} PDParameterList;

typedef struct PDSet
{
	DialogPtr dialog;
	PDParameterList paramList[MaxPNum];
	int paramNum;
	int textStart;
	struct PDSet *prev;
	struct PDSet *next;
} PDSet;

#ifndef SCLLocal
#include <VarHeader.h>
#include <ParametersDialog-v.c>
#else
#include "VarHeader.h"
#include "ParametersDialog-v.c"
#endif

/* Definition of PDSet has meen moved to ParametersDialog.c */

typedef struct PDSet *PDSetP;
typedef PDSetP *PDSetPP;

extern PDSetP s_paramDialogP;

#define SetUpParametersDialog() in_SetUpParametersDialogP(&s_paramDialogP)
#define SetUpParametersDialogP(d) in_SetUpParametersDialogP(d)
void in_SetUpParametersDialogP(PDSetP* s_paramDialogPP);

#define AddParameterP0(p,type,com,DpP) in_AddParameterP0((Ptr)p,type,com,DpP)
void in_AddParameterP0(Ptr p,int type,char* com,PDSetPP s_paramDialogP);
#define AddParameter(p,type,com) in_AddParameterP0((Ptr)p,type,com,&s_paramDialogP)

#define AddParameterP(p,type,com,Dp) in_AddParameterP((Ptr)p,type,com,Dp)
void in_AddParameterP(Ptr p,int type,char* com,PDSetP Dp);



#define ParametersDialog() ParametersDialogP(s_paramDialogP)
void ParametersDialogP(PDSetP p);
#if !S_UNIX || XDialog
short PDModalDialog(PDSet *pdp);
#endif
#if S_UNIX
void PDUnixShowPD(PDSet *pdp, int PanelNum);
int PDUnixDistinguishAlpha(char* reply, PDSet **pdpp, int *panelNumP);
void PDUnixHandleDigit(char *reply, PDSet *pdp, int panelNum);
#endif

#if S_UNIX
void PrintPDValues(int type, Ptr p);
#endif

#if !S_UNIX || XDialog
void SetPDValues(PDSetP s_paramDialog);
void GetPDValues(PDSetP s_paramDialog);
#endif /*!S_UNIX*/

#define DisplayParameters() DisplayParametersP(s_paramDialogP)
void DisplayParametersP(PDSetP p);


#define InitParametersDialog SetUpParametersDialog

void SavePDParameters(void);
void FprintfPDParamList(FILE *fp, PDParameterList paramList);
void PDParamListString(char *writeString, PDParameterList paramList);

void LoadPDParameters(void);
void LoadFileNamePDParameters(char *fileName);
int  FscanfPDParamList(FILE *fp, PDParameterList paramList);
int  PDStringToType(char *typeName);

void SetParametersDialogInitFile(char *fileName);
void PDInitLoadParams(void);

#endif /*pragma once*/
