/*
 *  s_Common.c
 *  Version 1.9.1 / 010423
 *  by Satoshi Takahashi and Yumiko Takahashi
 */

#ifndef S_COMMON_C /*pragma once*/
#define S_COMMON_C

#ifndef SCLLocal
#include <s_Config.h>
#else
#include "s_Config.h"
#endif
#if S_CARBON
#include <Carbon/Carbon.h>
#endif

/*#include <s_undef.c>*/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#if S_UNIX /* as s_Toolbox.h is not in parts-main */
#include <s_Toolbox.h>
#endif

#ifndef SCLLocal
#include <s_Common.h>
#include <StartSCL.h>
#include <s_RedrawFunc.h>
#include <s_String.h>
#include <s_QDGlobal.h>
#include <s_Carbon.h>
#include <s_CarbonMacro.h>
#else
#include "s_Common.h"
#include "StartSCL.h"
#include "s_RedrawFunc.h"
#include "s_String.h"
#include "s_QDGlobal.h"
#include "s_Carbon.h"
#include "s_CarbonMacro.h"
#endif

#if !S_UNIX /******************** !!!! UNIX ********************/

#ifndef SCLLocal
#include <s_undef.c>
#else
#include "s_undef.c"
#endif

PenState s_penState;

#endif /*!S_UNIX*/ /******************** UNIX ********************/

long s_longDummyString(char *s) {return 0;}
void s_voidDummyString(char* s) {}
void s_voidDummyVoid(void) {}


#if !S_UNIX
Str255 s_OOMMessage = "\pOut of Memory!";
#else
char* s_OOMMessage = "Out of Memory!\n";
#endif

void s_OOMError(void)
{
#if !S_UNIX

	s_SetGWorld(GetWindowPort(s_redrawWindow),s_display);
	Move(0,12);
	DrawString(s_OOMMessage);
	MoveTo(0,12);
	DrawString(s_OOMMessage);
	s_SetGWorld(GetWindowPort(s_textWindow),s_display);
	Move(0,12);
	DrawString(s_OOMMessage);
	MoveTo(0,12);
	DrawString(s_OOMMessage);
	FlushEvents(everyEvent,0);
	while(!GetNextEvent(mDownMask+keyDownMask,&s_event));
	ExitToShell();

#else /*!S_UNIX*/

	fprintf(stderr,s_OOMMessage);
	fflush(stderr);
	abort();

#endif /*else !S_UNIX*/
}

void PrintError(char* errorMessage, ...)
{
char s[256];
va_list ap;

	va_start(ap,errorMessage);
	vsprintf(s,errorMessage,ap);
	va_end(ap);
	s_printf(s);
	AlertMessage(s, 3);
	ExitToShell();
}


int PrintOSType(OSType type)
{
union sOSType
{
	OSType type;
	char   string[5];
}	conv;

	conv.type = type;
	conv.string[4] = 0;
	return s_printf("%s\n", conv.string);	
}


void AlertMessage(char *text, int level)
{
#if !S_UNIX
Str255 pText;

	C2PString(text, pText);
	ParamText(pText, 0, 0, 0);

	switch (level)
	{
	case 0: Alert(129, 0);
		break;
	case 1: NoteAlert(129, 0);
		break;
	case 2: CautionAlert(129, 0);
		break;
	case 3: StopAlert(129, 0);
		break;
	}
#else
	printf("Alert: %s\n", text);
#endif /*else S_UNIX*/
}


char s_c_defaultFont[256] = "Times";

int s_c_italicFlag = 1;

int s_c_subscriptLevel = 0;

typedef struct
{
	double subscriptRatio;
	double subscriptPos;
#if 0
	short sizeHistory[20];
#endif
} s_c_STType;

s_c_STType s_c_ST = 
{
	0.85, /*subscriptRatio*/
	0.2, /*subscriptPos*/
#if 0
	{12},
#endif
};

void DrawStyleChar(char c)
{
Style saveStyle; /* text syle(face) */

	if (s_c_italicFlag && isalpha(c))
	{
		saveStyle = GetPortTextFace(s_thePort);
		s_TextFace(italic);
		DrawChar(c);
		s_TextFace(saveStyle);
	}
	else
	{
		DrawChar(c);
	}
}

void DrawStyleText(char* text)
{
int curPos = 0;

	SetFont(s_c_defaultFont);
	DrawStyleTextIn(text, &curPos);
	SetFont(s_c_defaultFont);
}

void DrawStyleTextIn(char* text, int *curPosP)
{
int i;
char c;
char c2[3];

	i = *curPosP;
	c = text[i];
	
	while (c)
	{
		if (c & 0x80)
		{
			c2[0] = 2;
			c2[1] = text[i];
			c2[2] = text[i+1];
			DrawString((unsigned char *)c2);
			i++;
		}
		else
		{
			switch(c)
			{
			case '!': /*force print next char */
				c = text[++i];
				
				if (c)
					DrawStyleChar(c);
				else /* end of text */
					i--;
				
				break;
			case '$': /* $ is command string */
				StyleCommand(text, &i);
				break;

			case '_': /* _ means subscript */
				STSubscript(text, &i, 0);
				break;
		
			case '^': /* _ means superscript */
				STSubscript(text, &i, 1);
				break;
		
			case '}': /* end of subscript */
				if (s_c_subscriptLevel > 0)
				{
					*curPosP = i;
					return;
				}
				/* if not in subscript, continue */
				
			default:
				DrawStyleChar(c);
				break;
			}
		}
		c = text[++i];
	}
	*curPosP = i;
}

void StyleCommand(char *text, int *posP)
{
int i, j;
short size;
char commandString[64];
char c;

	i = *posP;

	j=++i;
	if (text[i]=='&') /* if next character is & it specifies the style */
	{
		switch(text[++i])
		{
			case 'I':
				s_c_italicFlag=1;
				break;
			case 'i':
				s_TextFace(italic);
				break;
			case 'B':
			case 'b':
				s_TextFace(bold);
				break;
			case 'P':
				s_c_italicFlag=0;
				break;
			case 'p':
			default:
				s_TextFace(0);
				break;
		}
	}
	else /* else, it specifies font name or size*/
	{
		while((c = text[++i]) && (c != '$'));
		strncpy(commandString, text+j, i-j);
		commandString[i-j] = 0;
		
		if (isalpha(commandString[0]))
		{
			SetFont(commandString);
		}
		else if (isdigit(commandString[0]))
		{
			size = atoi(commandString);
			s_TextSize(size);
		}
	}

	*posP = i;
}

void STSubscript(char *text, int *posP, int superScriptF)
{
short size;
int i;

	i = *posP;

	size = GetPortTextSize(s_thePort);
	
	if (!size) size = 12; /* default size is 12 */
	
#if 0
	s_c_ST.sizeHistory[s_c_subscriptLevel] = size;
#endif
	s_TextSize(size * s_c_ST.subscriptRatio + 0.5);
	if (superScriptF)
		s_Move(0, - (size*s_c_ST.subscriptPos+1));
	else
		s_Move(0, size*s_c_ST.subscriptPos+1);

	if (text[++i] == '{') /* _{ and } indicates subscript string */
	{
		s_c_subscriptLevel++;
		i++;
		DrawStyleTextIn(text, &i);
		STEndSubscript(superScriptF, size);
		s_TextSize(size);
	}
	else
	{
		DrawStyleChar(text[i]);
		if (superScriptF)
			s_Move(0, (size*s_c_ST.subscriptPos+1));
		else
			s_Move(0, - (size*s_c_ST.subscriptPos+1));
		s_TextSize(size);
	}
	
	*posP = i;
}

#if 1
void STEndSubscript(int superScriptF, short size)
{
//short size;

	s_c_subscriptLevel--;
#if 0
	size = s_c_ST.sizeHistory[s_c_subscriptLevel];
	s_TextSize(size);
#endif
	if (superScriptF)
		s_Move(0, (size*s_c_ST.subscriptPos+1));
	else
		s_Move(0, -(size*s_c_ST.subscriptPos+1));
}
#endif

void CenterString(char* string)
{
short stringWidth;
	stringWidth = TextWidth(string,0,strlen(string));
	s_Move(- stringWidth/2, 0);
}


int s_DirectDprintf(char *f, ...) /* print at current point */
{
extern char s_string[];
va_list ap;
int s_result;
#if defined(__MWERKS__)&&(__MWERKS__ >= 0x1100)&&MODIFYLF
char format[256];
#endif
	
	va_start(ap,f);

#if defined(__MWERKS__)&&(__MWERKS__ >= 0x1100)&&MODIFYLF
	LFToF(f, format);
	s_result = vsprintf(s_string,format,ap);
#else
	s_result = vsprintf(s_string,f,ap);
#endif

	va_end(ap);
	DrawText(s_string,0,strlen(s_string));
	return(s_result);
}


void SetPattern(Pattern pat, int pnumber)
{
	GetIndPattern(QDPattern(pat),0,pnumber);
}


void *CheckCalloc(size_t num, size_t count)
{
void *p;

	if (num == 0)
		num = 1;
		
	p = calloc(num, count);
	if (!p)
		s_OOMError();
	
	return p;
}
#endif /*pragma once*/
