/*
 *  s_Math.h
 *  Version 1.8.1 / 0001223
 *  by Satoshi Takahashi
 */

#ifndef S_MATH_C /*pragma once*/
#define S_MATH_C
#include <s_Config.h>

#include <math.h>
#include <s_Math.h>
#include <HugeArray.h>
#include <StartSCL.h>

#define min(a,b) (((a)>(b))?(b):(a))

double Binomial(int a,int b)
{
double x=1;
long i,j;

	b = min(b,(a-b));
	for(i=b,j=a; i>0; i--,j--)
	{
		x*=j;
		x/=i;
	}
	return x;
}

void MultiplyMatrix(int n,double **a,double **b,double **c) /* a, b and c can be same */
{
int i,j,k;
double **d;

	d = MakeArrayDouble2(n,n);
	for(i=0;i<n;i++)
	for(j=0;j<n;j++)
	for(k=0;k<n;k++)
		d[i][j] += a[i][k]*b[k][j];
	for(i=0;i<n;i++)
	for(j=0;j<n;j++)
		c[i][j] = d[i][j];
	FreeArrayDouble2(d,n,n);
}

double MaxEigenValue2(double a,double b,double c,double d)
{
double e,f;
	e = a + d;
	f = a*d - b*c;
	return ( e + sqrt(e*e - 4.0*f))/2.0;
}

double SafeLog(double x)
{
	if ( x > 0 )
		return log(x);
	else
	{
		s_printf("Error in SafeLog: %f is not positive\n", x);
		return 0;
	}
}


int NotADouble(double x)
{
    if ((x > 0) || (x <= 0))
        return 0;
    else
        return 1;
}

#endif /*pragme once*/
