/*
 *  s_Process.c
 *  Version 1.8.1 / 001201
 *  by Satoshi Takahashi
 *  most of codes are from UProcessUtils.p in ProcDoggie
 *  written by Forrest Tanaka
 *  copyright by Apple Computer
 */

#ifndef S_PROCESS_C /*pragma once*/
#define S_PROCESS_C

#include <s_Config.h>

#if S_CARBON
#include <Carbon/Carbon.h>
#endif

#include <string.h>
#include <stdlib.h>

#include <MoreFilesExtras.h>
#include <MoreDesktopMgr.h>

#include <s_String.h>
#if DEBUG_SCRIPT
#include <StartSCL.h>
#endif


#include <s_Process.h>
#include <VarBody.h>
/*#include <s_Process-v.c>*/


void LaunchOpen(FSSpec *appFileP, FSSpec *docFileP)
{
DocListHnd docList;
OSErr err;

	if (docFileP)
	{
		docList = CreateDocList();
		err = AddToDocList(docFileP, docList);
		err = LaunchProcApp(appFileP, docList, launchContinue);
	}
	else
		err = LaunchProcApp(appFileP, 0, launchContinue);
}



void Launch(FSSpec *appFileP)
{
OSErr err;

	err = LaunchProcApp(appFileP, 0, launchContinue);
}



DocListHnd CreateDocList(void)
{
DocListHnd newList; /*Handle to the new DocListRec*/

	newList = (DocListHnd)(NewHandle(sizeof(DocListRec)));
	if (newList)
	{
		newList[0]->docList = nil;
		newList[0]->openPrint = kOpenLaunch;
	}
	return newList;
}


Boolean IsEmptyDocList(DocListHnd theList)
{
	return (!theList) || (!theList[0]->docList);
}




OSErr AddToDocList (FSSpec *newFileP,
	                DocListHnd theList)
{

DocListEntryHnd newEntry, /*Handle to the new DocListEntryRec*/
				currEntry; /*Handle to DocListEntryRec being checked*/

	/* Create a new document list entry */
	newEntry = (DocListEntryHnd)(NewHandle(sizeof(DocListEntryRec)));
	if (newEntry)
	{
		/* Initialize the new document list entry */
		newEntry[0]->next = nil;
		newEntry[0]->docFile = *newFileP;

		if (IsEmptyDocList(theList))
			theList[0]->docList = newEntry;
		else
		{
			/* Start from the head of the list */
			currEntry = theList[0]->docList;

			/* Find the end of the list */
			while (currEntry[0]->next)
				currEntry = currEntry[0]->next;

			/* Attach the new entry to the end of the list */
			currEntry[0]->next = newEntry;
		}
		return noErr;
	}
	else
		return MemError();
}

OSErr LaunchProcApp (FSSpec *processFileP,
                     DocListHnd docList,
					 LaunchFlags options)
{
LaunchParamBlockRec launchParms;	/*Parameters for launching a file}*/
AEDesc				launchDesc;		/*Document file list descriptor*/
AppParametersPtr	appParms;		/*Pointer to the app parameters*/
OSErr				error;

	launchDesc.dataHandle = nil;

	if (docList)
	{
			error = CreateDocListDesc (docList, &launchDesc);
			HLock((Handle)(launchDesc.dataHandle));

			appParms = (AppParametersPtr)(*(launchDesc.dataHandle));
	}
	else
		appParms = nil;

	{
		launchParms.launchBlockID = extendedBlock;
		launchParms.launchEPBLength = extendedBlockLen;
		launchParms.launchFileFlags = 0;
		launchParms.launchControlFlags = options;
		launchParms.launchAppSpec = processFileP;
		launchParms.launchAppParameters = appParms;
	}
	return LaunchApplication (&launchParms);
}

OSErr CreateDocListDesc(DocListHnd theDocList,
	                            AEDesc *launchDescP)
{
const int kPutAtEnd = 0; /*For AEPutDesc; put new descriptor at end of list*/
AppleEvent			applMessage;	/*odoc/pdoc event w/ specd doc list*/
AEDescList			docDescList;	/*Desc list of doc alias descs*/
ProcessSerialNumber	selfPSN;		/*Our own PSN; not really used here*/
AEDesc				selfAddress;	/*Desc for our own PSN; ditto*/
AEDesc				docDesc;		/*Desc for doc alias*/
AliasHandle			docAlias;		/*Alias for specified docs*/
DocListEntryHnd		currDocEntry;	/*Handle to current doc list entry*/
AEEventID			openPrintCmd;	/*odoc or pdoc event?*/
Boolean				wasChanged;		/*TRUE if UpdateAlias changed alias*/
OSErr				error;


		selfAddress.dataHandle = nil;
		applMessage.dataHandle = nil;
		docDescList.dataHandle = nil;
		docAlias = nil;

		/* Make descriptor for my own PSN; just for AECreateAppleEvents yuks */
		selfPSN.highLongOfPSN = 0;
		selfPSN.lowLongOfPSN = kCurrentProcess;

		error = AECreateDesc(typeProcessSerialNumber, &selfPSN,
				sizeof(ProcessSerialNumber), &selfAddress);

		/* Create an AppleEvent for our list of document descriptors */
		if (theDocList[0]->openPrint == kOpenLaunch)
			openPrintCmd = kAEOpenDocuments;
		else if (theDocList[0]->openPrint == kPrintLaunch )
			openPrintCmd = kAEPrintDocuments;
		error = AECreateAppleEvent (kCoreEventClass, openPrintCmd, &selfAddress,
				kAutoGenerateReturnID, kAnyTransactionID, &applMessage);

		/* PSN copied into the odoc or pdoc event, so dont need it anymore */
		error = AEDisposeDesc (&selfAddress);

		/* Create list of descriptors for files; dont use list factorization */
		error = AECreateList (nil, 0, FALSE, &docDescList);

		/* Create an alias for the first document */
		currDocEntry = theDocList[0]->docList;
		HLock ((Handle)(currDocEntry));
		error = NewAlias (nil, &(currDocEntry[0]->docFile), &docAlias);
		HUnlock ((Handle)(currDocEntry));

		/* Put each document in the document list into document descriptor list */
		while (currDocEntry )
		{
			/* Convert alias into an alias descriptor manually */
			docDesc.descriptorType = typeAlias;
#if S_CARBON
			docDesc.dataHandle = (AEDataStorage)(docAlias);
#else
			docDesc.dataHandle = (Handle)(docAlias);
#endif

			/* Put the alias descriptor into the document descriptor list */
			error = AEPutDesc (&docDescList, kPutAtEnd, &docDesc);
			/* Go to the next document in the document list */
			currDocEntry = currDocEntry[0]->next;

			/* Convert the next documents FSSpec into the alias */
			if ( currDocEntry ) 
			{
				HLock ((Handle)(currDocEntry));
				error = UpdateAlias  (nil, &(currDocEntry[0]->docFile),
						docAlias, &wasChanged);
				HUnlock ((Handle)(currDocEntry));
			}
		}

		/* All aliases are in docDescList, so dont need the alias record */
		DisposeHandle ((Handle)(docAlias));
		docAlias = nil;

		/* Put the descriptor list of documents into the odoc or pdoc event */
		error = AEPutParamDesc (&applMessage, keyDirectObject, &docDescList);

		/* The descriptor list is copied into the odoc event, so get rid of it */
		error = AEDisposeDesc (&docDescList);

		/* Convert the odoc event to a descriptor suitable for the launch PB */
		error = AECoerceDesc (&applMessage, typeAppParameters, launchDescP);

		/* Were really interested in launchDesc, so dont need odoc message */
		error = AEDisposeDesc (&applMessage);

		return noErr;
}


int FindProcess(char *findName, ProcessSerialNumber *retPSN)
{
ProcessInfoRec procInfo;
Str255 procName;
char CprocName[255];
OSErr error;

	procInfo.processInfoLength = sizeof(ProcessInfoRec);
	procInfo.processName = procName;
	procInfo.processAppSpec = 0;

	retPSN->highLongOfPSN = 0;
	retPSN->lowLongOfPSN = kNoProcess;

	while (!GetNextProcess (retPSN))
	{
		error = GetProcessInformation (retPSN, &procInfo);
		P2CString(procName, CprocName);
#if DEBUG_SCRIPT
printf("%s#\n",findName);
printf("%s#\n",CprocName);
#endif
		if (!strcmp(CprocName,findName))
			return 1; /*found*/
	}
	return 0; /*not found*/
}



int FindProcessCreator(OSType creator, ProcessSerialNumber *retPSN)
{
ProcessInfoRec procInfo;
OSErr error;
Str255 procName;
#if DEBUG_SCRIPT
int i=0;
#endif


#if DEBUG_SCRIPT
PrintOSType(creator);
Stop();
#endif
	procInfo.processInfoLength = sizeof(ProcessInfoRec);
	procInfo.processName = procName;
	procInfo.processAppSpec = 0;

	retPSN->highLongOfPSN = 0;
	retPSN->lowLongOfPSN = kNoProcess;

	while (!GetNextProcess (retPSN))
	{
#if DEBUG_SCRIPT
s_printf("%d\n", i);
Stop();
i++;
#endif
		error = GetProcessInformation (retPSN, &procInfo);
		if (error)
			ExitToShell();
#if DEBUG_SCRIPT
PrintOSType(procInfo.processSignature);
Stop();
#endif
		if (procInfo.processSignature == creator)
			return 1; /*found*/
	}
	return 0; /*not found*/
}



int SPKillProcess(char *procName)
{
	return inSPKillProcess(procName, 0);
}


int SPKillProcessCreator(OSType procCreator)
{
	return inSPKillProcess(0, procCreator);
}


int inSPKillProcess(char *procName, OSType procCreator)
{
AEAddressDesc theDoomed;
AppleEvent quitEvent;
AppleEvent reply;
OSErr error;
ProcessSerialNumber theProcessNum;
int found;

	if (procName)
		found = FindProcess(procName, &theProcessNum);
	else
		found = FindProcessCreator(procCreator, &theProcessNum);		

#if DEBUG_SCRIPT
s_printf("%d\n", found);
Stop();
#endif
	if (found)
	{
		theDoomed.dataHandle = 0;
		quitEvent.dataHandle = 0;
		reply.dataHandle = 0;

		/* Create the Process Serial Number event descriptor */
		error = AECreateDesc(typeProcessSerialNumber, (Ptr)(&theProcessNum),
				sizeof(theProcessNum), &theDoomed);

		/* Create 'quit' event with the specified process serial number */
		error = AECreateAppleEvent(kCoreEventClass, kAEQuitApplication,
				&theDoomed, kAutoGenerateReturnID, kAnyTransactionID, &quitEvent);

		/* Send the 'quit' event */
		error = AESend (&quitEvent, &reply, kAENoReply,
				kAENormalPriority, kNoTimeOut, 0, 0);

		/* PSN in the AppleEvent, so can dispose of PSN descriptor */
		error = AEDisposeDesc (&theDoomed);

		/* Dispose of the 'quit' AppleEvent */
		error = AEDisposeDesc (&quitEvent);
		
		return 1;
	}
	
	return 0;
}

#if !S_CARBON
int SPKillOldProcess(char *procName)
{
ProcessSerialNumber procNum;
EvQEl *QueElemP;

	if (FindProcess(procName, &procNum))
	{
		SetFrontProcess(&procNum);

		PPostEvent(keyDown, 'q', &QueElemP);
		QueElemP->evtQModifiers = cmdKey;
		return 1;
	}
	
	return 0;
}
#endif


int LaunchOpenCreatorApp(OSType creator, FSSpec *docSpecP)
{
int error;

FSSpec appSpec;

	error = FindAppFromCreator(creator, &appSpec);

	if (!error)
		LaunchOpen(&appSpec, docSpecP);

	return error;
}

int FindAppFromCreator(OSType creator, FSSpec *appSpecP)
{
FSSpec volumes[20];
short volumeCount, volumeIndex = 1;
int i;

OSErr error = -1;

	OnLine(volumes, 20, &volumeCount, &volumeIndex);
	
	for(i = 0; i < volumeCount; i++)
	{
		error = FSpDTGetAPPL(0, volumes[i].vRefNum, creator, appSpecP);
		if (!error)
			break;
	}

	return error;
}

#endif /*pragma once*/