/*
 *  s_ToolboxTypes.h
 *  Version 2.0 / 010824
 *  by Satoshi Takahashi
 */

#ifndef S_TTOLBOXTYPES_H /*pragma once*/
#define S_TTOLBOXTYPES_H
#include <s_Config.h>
#if S_UNIX

#include <X11/Xlib.h>
/*dialog*/
#include <X11/Intrinsic.h> /* Boolean is defined here */


#define TBF 1 /* use two byte font */

typedef long OSStatus;

typedef struct
	{
		short top;
		short left;
		short bottom;
		short right;
	}	Rect;

typedef struct
	{
		short h;
		short v;
	}	Point;

typedef struct
	{
		unsigned short red;
		unsigned short green;
		unsigned short blue;
	}	RGBColor;

#define MAX_POLY 10
#define MAX_RGN 10
#define FONT_NUM 12
enum {plain, italic, bold, boldItalic};
typedef int Style;

typedef struct
	{
		Pixmap pixmap;
		int num;
	} Pattern[1];

extern Pattern white;
extern Pattern black;
extern Pattern gray;
extern Pattern ltGray;
extern Pattern dkGray;

typedef struct
	{
		Rect curRect;
		int polyUsed[MAX_POLY];
		int rgnUsed[MAX_RGN];
		char *(macFont[FONT_NUM]);
		char *(XFont[FONT_NUM]);
#if TBF
		int fontTwoByte[FONT_NUM];
#endif
		short fontNum;
		char fontStyle[32];
		int fontStyleNum;
		short fontSize;
		XColor basicColors[9];
	}	s_tbType;

typedef void *Handle;

typedef struct
	{
		/* SCL */
		int polyNum;
		/* X */
		int npoints;
		XPoint *points;
        /* QuickDraw */
        short polySize;
        Point *polyPoints;
	} Polygon;

typedef Polygon **PolyHandle;

#define POLY_SIZE_MAX 1024

typedef struct
{
	int num;
	XPoint savePoints[POLY_SIZE_MAX];
	PolyHandle poly;
} PolySaveType;

typedef int RgnHandle;


enum portKind
{
	portWindow,
	portDialog
};	

typedef struct GrafPort
	{
		/* Toolbox */
		Rect portRect;
		Point pnLoc;
		Point pnSize;
		int pnMode;
		Pattern pnPat;
		short pnVis;
		short txFont;
		short txSize;
		int txFace;
		Handle picSave;
		Handle rgnSave;
		PolySaveType *polySave;
		/* X */
		Display *display;
		GC gc;
		int screenNum;
		unsigned int borderWidth;
		Window win;
		unsigned long foreColor;
		unsigned long backColor;
		Colormap colormap;
		/* SCLXDialog */
		struct GrafPort *lastPort;
		int portKind;
	}	GrafPort;

typedef char(Str255)[256];

typedef struct GrafPort *GrafPtr;
typedef struct GrafPort *CGrafPtr;
typedef GrafPtr WindowPtr;


typedef struct
	{
		Point pnLoc;
		Point pnSize;
		int pnMode;
		Pattern pnPat;
	}	PenState;

enum { patCopy = GXcopy, notPatCopy = GXcopyInverted, patOr = GXor,
		notPatOr = GXorInverted, patXor = GXxor,
		notPatXor = GXequiv, patBic = GXandInverted,
		notPatBic = GXand } PatternMode;


extern s_tbType s_tb;
extern GrafPtr thePort;

typedef char GrafVerb;
enum { kQDGrafVerbFrame, kQDGrafVerbPaint, kQDGrafVerbErase,
        kQDGrafVerbInvert, kQDGrafVerbFill};


typedef void *Ptr;

enum EventMasks
{
	mDownMask=ButtonPressMask, mUpMask=ButtonReleaseMask,
	keyDownMask=KeyPressMask, keyUpMask=KeyReleaseMask,
	autoKeyMask=0, /*no corresponding mask in X*/
	updateMask=ExposureMask,
	diskMask=0,
	activMask=FocusChangeMask,
	everyEvent=-1L
};

enum EventTypes
{
	nullEvent, mouseDown, mouseUp, keyDown, keyUp, autoKey, updateEvt,
	diskEvt, activateEvt, networkEvt=10, driverEvt, app1Evt, app2Evt,
	app3Evt, app4Evt, osEvt=15, kHightLevelEvt=23
};

typedef struct EventRecord
{
	XEvent xEvent;
	short what;
	long message;
	long when;
	Point where;
	short modifiers;
} EventRecord;

#define nil ((void *)0)

#if 0 /*Boolean is defined in intrinsic.h */
typedef int Boolean;
#endif
#define false (0)
#define true (1)


enum { documentProc, dBoxProc, plainDBox, altDBoxProc,
	noGrowDocProc, movableDBoxProc,
	zoomDocProc = 8, zoomNoGrow = 12,
	rDocProc = 16};

typedef long OSType; 


/******************************************
	Parameters Dialog
	uses X Toolkit
*******************************************/


typedef Handle ControlHandle;
typedef Handle PicHandle;

typedef struct WindowRecord
{
	GrafPort port;
	short windowKind;
	char visible;
	char hilited;
	char goAwayFlag;
	char spareFlag;
	RgnHandle strucRgn;
	RgnHandle contRgn;
	RgnHandle updateRgn;
	Handle windowDefProc;
	Handle dataHandle;
	Str255 *titleHandle;
	short titleWidth;
	ControlHandle controlList;
	struct WindowRecord *nextWindow;
	PicHandle windowPic;
	long refCon;
} WindowRecord;

//typedef struct WindowRecord WindowRecord;

typedef WindowRecord *WindowPeek;


enum
{
	ctrlItem = 4,
	btnCtrl = 0, chkCtrl, radCtrl, resCtrl,
	statText = 8, editText = 16,
	iconItem = 32, picItem = 64, userItem = 0,
	itemDisable = 128,
	ok = 1, cancel = 2,
	stopIcon = 0, noteIcon = 1, cautionIcon = 2
};

enum
{
	kControlDialogItem			= ctrlItem,
	kButtonDialogItem			= ctrlItem + btnCtrl,
	kCheckBoxDialogItem			= ctrlItem + chkCtrl,
	kRadioButtonDialogItem		= ctrlItem + radCtrl,
	kResourceControlDialogItem	= ctrlItem + resCtrl,
	kStaticTextDialogItem		= statText,
	kEditTextDialogItem			= editText,
	kIconDialogItem				= iconItem,
	kPictureDialogItem			= picItem,
	kUserDialogItem				= userItem,
	kItemDisableBit				= itemDisable,
	kStdOkItemIndex				= ok,
	kStdCancelItemIndex			= cancel,
	kStopIcon					= stopIcon,
	kNoteIcon					= noteIcon,
	kCautionIcon				= cautionIcon
};

enum /*defined by ST for kResourceControlDialogItem*/
{
	rUpArrowDialogItem = 1,
	rDownArrowDialogItem,
	rSimpleEditTextDialogItem,
};

/* see "Adding Items ..." in Think Reference */
typedef struct DialogRecord *DialogPeek;

typedef struct itmDataType /*modified to use in Xt*/
{
	long value;
	char label[256];
} itmDataType;


typedef struct DITLItem
{
	short resID;
	short itemN;
	Handle itmHndl;
	Rect itmRect;
	char itmType;
	itmDataType itmData;
	/* added for SCLX */
	int inited;
	Widget widget;
	DialogPeek dialog;
	int NCFlag;
	Widget *optWidget;
} DITLItem;

typedef struct ItemList
{
	short dlgMaxIndex;
	DITLItem DITLItems[1];
} ItemList;

typedef struct EditButtonPopupRec
{
	Widget shell;
	Widget form;
	Widget text;
	Widget button;
	DITLItem *dItem;
	int appear;
} SXEditButtonPopupRec, *SXEditButtonPopupPtr;


typedef struct s_XtType
{
	Widget topWidget;
	XtAppContext appContext;
	short itemHit;
	Pixmap checkMark;
	SXEditButtonPopupPtr editButtonPopup;
	Pixmap upArrow;
	Pixmap downArrow;
	Widget radioGroup[16]; /* group 0 will not be used */
} s_XtType;

extern s_XtType s_Xt;


typedef void *TEHandle;

typedef struct DialogRecord
{
	WindowRecord window;
	Handle items;
	TEHandle textH;
	short editField;
	short editOpen;
	short aDefItem;
/* Used for X */
	short itemCount;
	short recordNumber;
	Widget shell;
	Widget form;
	short resID;
} DialogRecord;

//typedef DialogRecord *DialogPeek;

typedef WindowPtr DialogPtr;

typedef short (ProcPtr)();

#endif /*S_UNIX*/
#endif /*pragma once*/
