/*
 *  s_UNIX.c
 *  Version 1.7 / 980918
 *  by Satoshi Takahashi
 */

#ifndef S_UNIX_C /*pragma once*/
#define S_UNIX_C

#include <s_Config.h>
#include <s_Debug.h>

/* Common(also in Mac) */

#if S_UNIX
#include <unistd.h>
#if SunOS || Linux
#include <termio.h>
#endif
#include <termios.h>

#if FreeBSD
#include <sys/ioctl.h>
#endif
#endif /*S_UNIX*/

void SetBackspace(void)
{
struct termios term;
#if S_UNIX
#if SunOS || Linux
        ioctl(0, TCGETS, &term);
        term.c_cc[VERASE]=8;
        ioctl(0, TCSETS, &term);
#endif
#if FreeBSD
        ioctl(0, TIOCGETA, &term);
        term.c_cc[VERASE]=8;
        ioctl(0, TIOCSETA, &term);
#endif
#endif /*S_UNIX*/
}


/* only for UNIX */

#if S_UNIX

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <s_UNIX.h>
#include <s_Toolbox.h>

#if OLD_C
#include <OldC.h>
#endif

#include <s_undef.c>

#if OLD_C
#define atexit s_atexit
#endif /*OLD_C*/


#include <VarBody.h>
#include <s_UNIX-v.c>

extern GrafPtr s_drawingWindow;

void UNIXStart(void)
{
char fileName[256];

dp1("UNIXStart\n");

#if Linux
	s_UNIX.printfFile = stdout;
#endif

	atexit(&UNIXExit);

	if (!s_Config.foreground)
	{
		sprintf(fileName, "%sprintf_output", s_Config.dataDirNameString);
		s_UNIX.printfFile = fopen(fileName, "w");
	}
	
	sprintf(fileName, "%sSCLDrawingData.c", s_Config.dataDirNameString);
	s_UNIX.drawingFile = fopen(fileName, "w");
	
	if (s_UNIX.drawingFile)
	{
		s_UNIX.drawingFlag = 1;
		fprintf(s_UNIX.drawingFile, "#include <SCL.h>\n");
		fprintf(s_UNIX.drawingFile, "Rect r={0,0,0,0};\n");
		fprintf(s_UNIX.drawingFile, "PolyHandle poly[%d];\n",MAX_POLY);
		fprintf(s_UNIX.drawingFile, "RgnHandle rgn[%d];\n",MAX_RGN);
		fprintf(s_UNIX.drawingFile, "RGBColor color={0,0,0};\n");
		fprintf(s_UNIX.drawingFile, "short fNum;\n");
		fprintf(s_UNIX.drawingFile, "Pattern pat;\n");
		fprintf(s_UNIX.drawingFile, "main()\n");
		fprintf(s_UNIX.drawingFile, "{\n");
	}
	
	s_UNIX.sec = time(0);
	s_UNIX.stopSec = s_UNIX.sec + s_UNIX.stopInterval;

	if (getenv("BACKSPACE"))
		SetBackspace();
dp1("End: UNIXStart\n");
}

void UNIXExit(void)
{
	if (s_UNIX.drawingFlag)
	{
		fprintf(s_UNIX.drawingFile, "Stop();\n");
		fprintf(s_UNIX.drawingFile, "}\n");
		fclose(s_UNIX.drawingFile);
	}

	if (!s_Config.foreground)
	{
		fclose(s_UNIX.printfFile);
	}
}

int CancelName(char* filename)
{
	
	if((filename[0] == 'c') || (filename[0] == 'C'))
	{
		if (strlen(filename) == 1)
			return 1;
		if (!strcmp(filename, "cancel"))
			return 1;
		if (!strcmp(filename, "CANCEL"))
			return 1;
	}
	if((filename[0] == 'n') || (filename[0] == 'N'))
	{
		if (strlen(filename) == 1)
			return 1;
		if (!strcmp(filename, "no"))
			return 1;
		if (!strcmp(filename, "NO"))
			return 1;
	}
	return 0;
}




#endif /*S_UNIX*/

#endif /*pragma once*/
